USE [GestorGastosBD]
GO

DROP PROCEDURE [dbo].[prcDeleteUsuariosSupervisados]
GO

CREATE PROCEDURE [dbo].[prcDeleteUsuariosSupervisados]
	@IdUsuario int
AS
BEGIN
	DELETE UsuarioSuperiorRel
	WHERE IdUsuarioSuperior = @IdUsuario
END
GO

--------------------------------------------------------------------------------------------------------------

DROP PROCEDURE [dbo].[prcDeleteUsuarioSupervisor]
GO

CREATE  PROCEDURE [dbo].[prcDeleteUsuarioSupervisor]
	@IdUsuario int
AS
BEGIN
	DELETE FROM UsuarioSuperiorRel WHERE IdUsuario = @IdUsuario
END
GO

--------------------------------------------------------------------------------------------------------------

DROP PROCEDURE [dbo].[prcGetHojasSupervisar]
GO

CREATE PROCEDURE [dbo].[prcGetHojasSupervisar]
(
	@IdUsuarioSupervisor INT = NULL,
	@IdHojaEstadoTipo INT = NULL,
	@NIF VARCHAR(15) = NULL,
	@NombreCompleto VARCHAR(150) = NULL
)
AS
BEGIN
	DECLARE @sql VARCHAR(1000)
	DECLARE @fechaLimite VARCHAR(50)

	SET @fechaLimite = CONVERT(varchar(4), YEAR(GETDATE())) + '-' + convert(varchar(2), MONTH(GETDATE())) + '-1 00:00:00.000' 

	SET @sql  = 'SELECT 
				Hoja.IdHoja, 
				Hoja.Referencia, 
				Hoja.IdHojaEstadoTipo, 
				Hoja.Importe,
				Hoja.FechaCreacion, 
				Hoja.IdUsuarioCreador, 
				Hoja.FechaSupervision, 
				Hoja.IdUsuarioSupervisor,
				Hoja.IndBaja, 
				Hoja.DesMotivoDenegada
				FROM Hoja
				LEFT JOIN UsuarioSuperiorRel ON UsuarioSuperiorRel.IdUsuario = Hoja.IdUsuarioCreador  
				INNER JOIN Usuario ON Usuario.IdUsuario = Hoja.IdUsuarioCreador
				WHERE Hoja.IndBaja = 0
				AND Hoja.FechaCreacion < CONVERT(DATETIME,''' + @fechaLimite + ''', 121)'

	IF(@IdUsuarioSupervisor IS NOT NULL) 
		SET @sql = @sql + ' AND UsuarioSuperiorRel.IdUsuarioSuperior = ' + CAST(@IdUsuarioSupervisor AS VARCHAR(8))  
	ELSE
		SET @sql = @sql + ' AND UsuarioSuperiorRel.IdUsuarioSuperior IS NULL' 

	IF(@IdHojaEstadoTipo IS NOT NULL) 
		SET @sql = @sql + ' AND Hoja.IdHojaEstadoTipo = ' + CAST(@IdHojaEstadoTipo AS VARCHAR(5)) 
	IF(@NIF IS NOT NULL AND @NIF <> '') 
		SET @sql = @sql + ' AND Usuario.NIF = ''' + @NIF + ''''
	IF(@NombreCompleto IS NOT NULL AND @NombreCompleto <> '') 
		SET @sql = @sql + ' AND Usuario.NombreCompleto LIKE ''%' + @NombreCompleto + '%''' 

	exec (@sql)
END
GO

--------------------------------------------------------------------------------------------------------------

DROP PROCEDURE [dbo].[prcGetHojaVigente]
GO

CREATE PROCEDURE [dbo].[prcGetHojaVigente]
	@IdUsuarioCreador int
AS
BEGIN
	SELECT 
	Hoja.IdHoja,
	Hoja.Referencia,
	Hoja.IdHojaEstadoTipo,
	Hoja.Importe,
	Hoja.FechaCreacion,
	Hoja.IdUsuarioCreador,
	Hoja.FechaSupervision,
	Hoja.IdUsuarioSupervisor,
	Hoja.IndBaja,
	Hoja.DesMotivoDenegada
	FROM Hoja
	WHERE IndBaja = 0
	AND IdUsuarioCreador = @IdUsuarioCreador
	AND YEAR(FechaCreacion) = YEAR(GETDATE())
	AND MONTH(FechaCreacion) = MONTH(GETDATE())
END
GO

--------------------------------------------------------------------------------------------------------------

DROP PROCEDURE [dbo].[prcInsertUsuarioSupervisor]
GO

CREATE  PROCEDURE [dbo].[prcInsertUsuarioSupervisor]
	@IdUsuarioSuperior INT,
	@IdUsuario int
AS
BEGIN
	INSERT INTO UsuarioSuperiorRel 
	VALUES (@IdUsuario, @IdUsuarioSuperior)
END
GO

--------------------------------------------------------------------------------------------------------------

DROP PROCEDURE [dbo].[prcUpdateHojaImporte]
GO

CREATE PROCEDURE [dbo].[prcUpdateHojaImporte]
	@IdHoja int
AS
BEGIN
	UPDATE Hoja
	SET Importe = (SELECT SUM(Gasto.Unidades * Gasto.ImporteUnidad) FROM Gasto WHERE Hoja.IdHoja = Gasto.IdHoja GROUP BY IdHoja)
	WHERE IdHoja = @IdHoja
END
GO

--------------------------------------------------------------------------------------------------------------

DROP PROCEDURE [dbo].[prcUpdateHojasBaja]
GO

CREATE PROCEDURE [dbo].[prcUpdateHojasBaja]
(
	@IdUsuario INT,
	@IndBaja  BIT 
)
AS
BEGIN
	UPDATE Hoja
	SET IndBaja = @IndBaja
	WHERE IdUsuarioCreador = @IdUsuario
END
GO

--------------------------------------------------------------------------------------------------------------

DROP PROCEDURE [dbo].[prcUpdateUsuarioSupervisor]
GO

CREATE PROCEDURE [dbo].[prcUpdateUsuarioSupervisor]
	@IdUsuarioSuperior INT,
	@IdUsuario int
AS
BEGIN
	UPDATE UsuarioSuperiorRel
	SET IdUsuarioSuperior = @IdUsuarioSuperior
	WHERE IdUsuario = @IdUsuario
END
GO





