﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace GestorGastos.Administracion
{
    public class Common
    {

        public static bool ValidarEmail(string valor)
        {
            return Regex.IsMatch(valor, @"\A(?:[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?)\Z");
        }

        /// <summary>
        /// Validación de un NIF
        /// </summary>
        /// <param name="pNIF"></param>
        /// <returns></returns>
        public static bool EsCorrectoNIF(string pNIF)
        {
            if (!EstaInformado(pNIF))
                return false;

            if (!EsValidarLongitudEstrictaIgualQue(pNIF, 9))
                return false;

            pNIF = pNIF.ToUpper();

            Regex oRegex = new Regex(@"\b[0-9XYZ]{1}\d{7}\B[A-Z]{1}");
            if (oRegex.IsMatch(pNIF))
            {
                try
                {
                    //para NIF español
                    string letraNif = pNIF.Trim().Substring(pNIF.Length - 1, 1);
                    int numNif = 0;

                    //Para NIFs extranjeros (NIE)
                    if (pNIF.StartsWith("X") || pNIF.StartsWith("Y") || pNIF.StartsWith("Z"))
                    {
                        string digitosNIF = string.Empty;

                        if (pNIF.StartsWith("X"))
                            digitosNIF = "0";
                        else if (pNIF.StartsWith("Y"))
                            digitosNIF = "1";
                        else if (pNIF.StartsWith("Z"))
                            digitosNIF = "2";

                        //concatenar digito con el resto de digitos del pNIF (8)
                        digitosNIF = digitosNIF.Insert(1, pNIF.Trim().Substring(1, pNIF.Length - 2));
                        numNif = Int32.Parse(digitosNIF);

                        if (digitosNIF.Length != 8)
                            return false;
                    }
                    else
                        numNif = Int32.Parse(pNIF.Trim().Substring(0, pNIF.Length - 1));

                    string[] letraControl = {"T", "R", "W", "A", "G", "M", "Y", "F",
                                                                 "P", "D", "X", "B", "N", "J", "Z", "S",
                                                                 "Q", "V", "H", "L", "C", "K", "E"};

                    int indiceLetraControl = (numNif % 23);

                    if (letraNif.Equals(letraControl[indiceLetraControl]))
                        return true;
                    else
                        return false;
                }
                catch
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        private static bool EstaInformado(string valor)
        {
            return !(string.IsNullOrEmpty(valor));
        }

        private static bool EsValidarLongitudEstrictaIgualQue(string valor, int longitud)
        {
            if (valor == null) return false;
            if (valor.Trim().Length == longitud) return true;
            return false;
        }

        /// <summary>
        /// La contraseña debe tener 6 dígitos 
        /// </summary>
        /// <param name="password"></param>
        /// <returns></returns>
        public static bool ValidarContrasena(string password)
        {
            if (string.IsNullOrEmpty(password))
                return false;
            else
            {
                if (password.Length != 6)
                    return false;
            }
            return true;
        }

        /// <summary>
        /// El nombre de usuario como máximo tendrá 10 dígitos
        /// </summary>
        /// <param name="username"></param>
        /// <returns></returns>
        public static bool ValidarNombreUsuario(string username)
        {
            if (string.IsNullOrEmpty(username))
                return false;
            else
            {
                if (username.Length > 10)
                    return false;
            }
            return true;
        }
    }
}
