﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Resources;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using GestorGastos.Administracion.GestorGastosService;

namespace GestorGastos.Administracion
{
    public partial class FormCalendario : FormBase
    {
        GestorGastosServiceClient servicioGG = new GestorGastosServiceClient();

        public FormCalendario()
        {
            InitializeComponent();
            dtgCalendario.AutoGenerateColumns = false;
        }

        private void FormCalendario_Load(object sender, EventArgs e)
        {
            ActualizarDataGrid();
        }

        private void btnGuardar_Click(object sender, EventArgs e)
        {
            try
            {
                DateTime fecha = dtpFecha.Value.Date;

                if (fecha.DayOfWeek == DayOfWeek.Saturday || fecha.DayOfWeek == DayOfWeek.Sunday)
                    this.MostrarInformacion("No es necesario añadir la fecha ya que es fin de semana y ya se considera festivo.");
                else
                {
                    bool resultOK = servicioGG.AgregarFestividad(fecha);
                    if (resultOK)
                    {
                        ActualizarDataGrid();
                        this.MostrarInformacion("Fecha agregada correctamente.");
                    }
                    else
                        this.MostrarError("La fecha ya estaba agregada como festiva.");
                }

                dtpFecha.Value = DateTime.Today;
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error en la actualización del calendario.\n" + ex.Message);
            }
        }

        private void ActualizarDataGrid()
        {
            try
            {
                List<CalendarioDTO> listaFestivos = servicioGG.ObtenerFestividades();
                dtgCalendario.DataSource = listaFestivos;
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error en la actualización del calendario.\n" + ex.Message);
            }
        }

        private void dtgCalendario_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex >= 0 && this.dtgCalendario.Columns[e.ColumnIndex].Name == "Eliminar" && e.RowIndex >= 0)
            {
                DateTime fecha = (DateTime)this.dtgCalendario.Rows[e.RowIndex].Cells["FechaFestiva"].Value;

                DialogResult result = MessageBox.Show("¿Desea eliminar la fecha " + fecha.ToString() + "?", "Eliminar", MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
                if (result.Equals(DialogResult.OK))
                {
                    int idCalendario = (int)this.dtgCalendario.Rows[e.RowIndex].Cells["IdCalendario"].Value;
                    try
                    {
                        servicioGG.EliminarFecha(idCalendario);
                        ActualizarDataGrid();
                        this.MostrarInformacion("Fecha eliminada correctamente.");
                    }
                    catch (Exception ex)
                    {
                        this.MostrarError("Se ha producido un error:\n" + ex.Message);
                    }
                }
            }
        }

        private void dtgCalendario_CellPainting(object sender, DataGridViewCellPaintingEventArgs e)
        {
            try
            {
                if (e.ColumnIndex >= 0 && this.dtgCalendario.Columns[e.ColumnIndex].Name == "Eliminar" && e.RowIndex >= 0)
                {
                    e.Paint(e.CellBounds, DataGridViewPaintParts.All);
                    DataGridViewButtonCell celBoton = this.dtgCalendario.Rows[e.RowIndex].Cells["Eliminar"] as DataGridViewButtonCell;
                    Image imgDelete = Properties.Resources.No_check;
                    e.Graphics.DrawImage(imgDelete, e.CellBounds.Left + 3, e.CellBounds.Top + 3, 20, 20);
                    this.dtgCalendario.Rows[e.RowIndex].Height = 30;
                    this.dtgCalendario.Columns[e.ColumnIndex].Width = 30;
                    e.Handled = true;
                }
            }
            catch { }
        }

        private void btnSalir_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}


