﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using GestorGastos.Administracion.GestorGastosService;

namespace GestorGastos.Administracion
{
    public partial class FormConfiguracion : FormBase
    {
        GestorGastosServiceClient servicioGG = new GestorGastosServiceClient();

        int? idGastoTipoSelected;
        int? idDepartementoSelected;
        int? idCategoriaProfesionalSelected;

        public FormConfiguracion()
        {
            InitializeComponent();
            dtgGastosTipo.AutoGenerateColumns = false;
            dtgDepartamentos.AutoGenerateColumns = false;
            dtgCategorias.AutoGenerateColumns = false;
        }

        private void FormConfiguracion_Load(object sender, EventArgs e)
        {
            ActualizarGestionGastosTipo();
            ActualizarGestionDepartamentos();
            ActualizarGestionCategorias();
        }

        #region Tipos de gasto
        private void ActualizarGestionGastosTipo()
        {
            try
            {
                List<GastoTipoDTO> listaGastosTipo = servicioGG.ObtenerGastosTipos();
                dtgGastosTipo.DataSource = listaGastosTipo;
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error:" + ex.Message);
            }

            pnlEdicionGasto.Visible = false;
        }

        private void dtgGastosTipo_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                DataGridViewRow filaSeleccionada = dtgGastosTipo.Rows[e.RowIndex];
                idGastoTipoSelected = (int)filaSeleccionada.Cells["IdGastoTipo"].Value;

                CargarPanelEdicionGastoTipo(idGastoTipoSelected);
            }
        }

        private void btnNuevoGastoTipo_Click(object sender, EventArgs e)
        {
            idGastoTipoSelected = null;
            CargarPanelEdicionGastoTipo(idGastoTipoSelected);
        }

        private void CargarPanelEdicionGastoTipo(int? idGastoTipoSelected)
        {
            pnlEdicionGasto.Visible = true;
            lblDescripcionGastoTipoError.Visible = false;
            lblImporteError.Visible = false;
            lblImporteFestivoError.Visible = false;

            if (idGastoTipoSelected.HasValue)
            {
                try
                {
                    GastoTipoDTO gastosTipo = servicioGG.ObtenerGastoTipo(idGastoTipoSelected.Value);
                    if (gastosTipo != null)
                    {
                        txtDescripcionGastoTipo.Text = gastosTipo.DescGastoTipo;
                        chkImporteFijo.Checked = !gastosTipo.IndImporteLibre.GetValueOrDefault();
                        chkImporteDiferenteFestivos.Checked = gastosTipo.IndImporteDiferenteFestivo.GetValueOrDefault();
                        chkAdjuntarFichero.Checked = gastosTipo.IndAdjuntarFichero.GetValueOrDefault();
                        txtImporte.Text = gastosTipo.Importe.ToString();
                        txtImporteFestivo.Text = gastosTipo.ImporteFestivo.ToString();

                        chkImporteDiferenteFestivos.Enabled = chkImporteFijo.Checked;
                        txtImporte.Enabled = chkImporteFijo.Checked;
                        txtImporteFestivo.Enabled = chkImporteFijo.Checked && chkImporteDiferenteFestivos.Checked;
                    }
                }
                catch (Exception ex)
                {
                    this.MostrarError("Se ha producido un error:" + ex.Message);
                }
            }
            else
                LimpiarPanelEdicionGastoTipo();

            //Deshabilitar el resto
            pnlDepartamentos.Enabled = false;
            pnlCategorias.Enabled = false;
        }

        private void LimpiarPanelEdicionGastoTipo()
        {
            txtDescripcionGastoTipo.Text = string.Empty;
            chkImporteFijo.Checked = false;
            chkImporteDiferenteFestivos.Checked = false;
            chkImporteDiferenteFestivos.Enabled = false;
            chkAdjuntarFichero.Checked = false;
            txtImporte.Text = string.Empty;
            txtImporteFestivo.Text = string.Empty;
            txtImporte.Enabled = false;
            txtImporteFestivo.Enabled = false;

            //Habilitar el resto
            pnlDepartamentos.Enabled = true;
            pnlCategorias.Enabled = true;
        }

        private void chkImporteFijo_CheckedChanged(object sender, EventArgs e)
        {
            chkImporteDiferenteFestivos.Enabled = chkImporteFijo.Checked;
            if (!chkImporteFijo.Checked)
                chkImporteDiferenteFestivos.Checked = false;
            txtImporte.Enabled = chkImporteFijo.Checked;
            txtImporteFestivo.Enabled = chkImporteFijo.Checked && chkImporteDiferenteFestivos.Checked;
            if (!txtImporte.Enabled)
                txtImporte.Text = string.Empty;
            if (!txtImporteFestivo.Enabled)
                txtImporteFestivo.Text = string.Empty;
        }

        private void chkImporteDiferenteFestivos_CheckedChanged(object sender, EventArgs e)
        {
            txtImporteFestivo.Enabled = chkImporteDiferenteFestivos.Checked;
            if (!txtImporteFestivo.Enabled)
                txtImporteFestivo.Text = string.Empty;
        }

        private void btnCancelarGastoTipo_Click(object sender, EventArgs e)
        {
            pnlEdicionGasto.Visible = false;
            LimpiarPanelEdicionGastoTipo();
        }

        private void btnAceptarGastoTipo_Click(object sender, EventArgs e)
        {
            GastoTipoDTO gastoTipo = new GastoTipoDTO();
            gastoTipo.DescGastoTipo = txtDescripcionGastoTipo.Text.Trim();
            gastoTipo.IdGastoTipo = idGastoTipoSelected.GetValueOrDefault();

            decimal importe;
            if (!string.IsNullOrEmpty(txtImporte.Text))
            {
                string importeTexto = txtImporte.Text.Replace('.', ',');
                importeTexto = String.Format("{0:0.00}", importeTexto);
                if (decimal.TryParse(importeTexto, out importe))
                    gastoTipo.Importe = importe;
            }
            else gastoTipo.Importe = null;

            if (!string.IsNullOrEmpty(txtImporteFestivo.Text))
            {
                string importeFestivoTexto = txtImporteFestivo.Text.Replace('.', ',');
                importeFestivoTexto = String.Format("{0:0.00}", importeFestivoTexto);
                if (decimal.TryParse(importeFestivoTexto, out importe))
                    gastoTipo.ImporteFestivo = importe;
            }
            else
                gastoTipo.ImporteFestivo = null;

            gastoTipo.IndAdjuntarFichero = chkAdjuntarFichero.Checked;
            gastoTipo.IndImporteDiferenteFestivo = chkImporteDiferenteFestivos.Checked;
            gastoTipo.IndImporteLibre = !chkImporteFijo.Checked;

            if (ValidarDatosGastoTipo(gastoTipo))
            {
                try
                {
                    servicioGG.ActualizarGastoTipo(gastoTipo);
                    ActualizarGestionGastosTipo();
                    MessageBox.Show("Tipo de gasto actualizados correctamente.", "Tipos de gasto");
                }
                catch (Exception ex)
                {
                    this.MostrarError("Se ha producido un error:" + ex.Message);
                }
            }
            else
            {
                this.MostrarInformacion("No se han informado los datos correctamente.");
            }

            //Habilitar el resto
            pnlDepartamentos.Enabled = true;
            pnlCategorias.Enabled = true;
        }

        private bool ValidarDatosGastoTipo(GastoTipoDTO gastoTipo)
        {
            bool validado = true;

            lblDescripcionGastoTipoError.Visible = string.IsNullOrEmpty(gastoTipo.DescGastoTipo);
            if (lblDescripcionGastoTipoError.Visible)
                validado = false;

            lblImporteError.Visible = (!gastoTipo.IndImporteLibre.GetValueOrDefault() && !gastoTipo.Importe.HasValue);
            if (lblImporteError.Visible)
                validado = false;

            lblImporteFestivoError.Visible = (gastoTipo.IndImporteDiferenteFestivo.GetValueOrDefault() && !gastoTipo.ImporteFestivo.HasValue);
            if (lblImporteFestivoError.Visible)
                validado = false;

            return validado;
        }

        private void dtgGastosTipo_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex >= 0 && this.dtgGastosTipo.Columns[e.ColumnIndex].Name == "Eliminar")
            {
                string descGastoTipo = this.dtgGastosTipo.Rows[e.RowIndex].Cells["DescGastoTipo"].Value.ToString();

                DialogResult result = MessageBox.Show("¿Desea eliminar el tipo de gasto \"" + descGastoTipo + "\"?", "Eliminar", MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
                if (result.Equals(DialogResult.OK))
                {
                    int idGastoTipo = (int)this.dtgGastosTipo.Rows[e.RowIndex].Cells["IdGastoTipo"].Value;
                    try
                    {
                        servicioGG.EliminarGastoTipo(idGastoTipo);
                        ActualizarGestionGastosTipo();
                        MessageBox.Show("Tipo de gasto eliminado correctamente.", "Tipos de gasto");
                    }
                    catch (Exception ex)
                    {
                        this.MostrarError("Se ha producido un error:" + ex.Message);
                    }
                }
            }
        }

        private void dtgGastosTipo_CellPainting(object sender, DataGridViewCellPaintingEventArgs e)
        {
            try
            {
                if (e.ColumnIndex >= 0 && this.dtgGastosTipo.Columns[e.ColumnIndex].Name == "Eliminar" && e.RowIndex >= 0)
                {
                    e.Paint(e.CellBounds, DataGridViewPaintParts.All);
                    DataGridViewButtonCell celBoton = this.dtgGastosTipo.Rows[e.RowIndex].Cells["Eliminar"] as DataGridViewButtonCell;
                    Image imgDelete = Properties.Resources.No_check;
                    e.Graphics.DrawImage(imgDelete, e.CellBounds.Left + 3, e.CellBounds.Top + 3, 20, 20);
                    this.dtgGastosTipo.Rows[e.RowIndex].Height = 30;
                    this.dtgGastosTipo.Columns[e.ColumnIndex].Width = 30;
                    e.Handled = true;
                }
            }
            catch { }
        }
        #endregion Tipos de gasto

        #region Departamentos
        private void ActualizarGestionDepartamentos()
        {
            try
            {
                List<DepartamentoDTO> listaDepartametos = servicioGG.ObtenerDepartamentos();
                dtgDepartamentos.DataSource = listaDepartametos;
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error:" + ex.Message);
            }

            pnlEdicionDepartamento.Visible = false;
        }

        private void dtgDepartamentos_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                DataGridViewRow filaSeleccionada = dtgDepartamentos.Rows[e.RowIndex];
                idDepartementoSelected = (int)filaSeleccionada.Cells["IdDepartamento"].Value;

                CargarPanelEdicionDepartamento(idDepartementoSelected);
            }
        }

        private void CargarPanelEdicionDepartamento(int? idDepartementoSelected)
        {
            pnlEdicionDepartamento.Visible = true;
            lblDescripcionDepartamentoError.Visible = false;

            if (idDepartementoSelected.HasValue)
            {
                try
                {
                    DepartamentoDTO departamento = servicioGG.ObtenerDepartamento(idDepartementoSelected.Value);
                    if (departamento != null)
                    {
                        txtDescripcionDepartamento.Text = departamento.DescDepartamento;
                    }
                }
                catch (Exception ex)
                {
                    this.MostrarError("Se ha producido un error:" + ex.Message);
                }
            }
            else
                LimpiarPanelEdicionDepartamento();

            //Deshabilitar el resto
            pnlGastoTipo.Enabled = false;
            pnlCategorias.Enabled = false;
        }

        private void LimpiarPanelEdicionDepartamento()
        {
            txtDescripcionDepartamento.Text = string.Empty;
           
            //Habilitar el resto
            pnlGastoTipo.Enabled = true;
            pnlCategorias.Enabled = true;
        }

        private void btnNuevoDepartamento_Click(object sender, EventArgs e)
        {
            idDepartementoSelected = null;
            CargarPanelEdicionDepartamento(idDepartementoSelected);
        }

        private void btnCancelarDepartamento_Click(object sender, EventArgs e)
        {
            pnlEdicionDepartamento.Visible = false;
            LimpiarPanelEdicionDepartamento();
        }

        private void btnAceptarDepartamento_Click(object sender, EventArgs e)
        {
            DepartamentoDTO departamento = new DepartamentoDTO();
            departamento.DescDepartamento = txtDescripcionDepartamento.Text;
            departamento.IdDepartamento = idDepartementoSelected.GetValueOrDefault();

            if (ValidarDatosDepartamento(departamento))
            {
                try
                {
                    servicioGG.ActualizarDepartamento(departamento);
                    ActualizarGestionDepartamentos();
                    MessageBox.Show("Departamentos actualizados correctamente.", "Departamentos");
                }
                catch (Exception ex)
                {
                    this.MostrarError("Se ha producido un error:" + ex.Message);
                }
            }
            else
            {
                this.MostrarInformacion("No se han informado los datos correctamente.");
            }

            //Habilitar el resto
            pnlGastoTipo.Enabled = true;
            pnlCategorias.Enabled = true;
        }

        private bool ValidarDatosDepartamento(DepartamentoDTO departamento)
        {
            bool validado = true;

            lblDescripcionDepartamentoError.Visible = string.IsNullOrEmpty(departamento.DescDepartamento);
            if (lblDescripcionDepartamentoError.Visible)
                validado = false;

            return validado;
        }

        private void dtgDepartamentos_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex >= 0 && this.dtgDepartamentos.Columns[e.ColumnIndex].Name == "EliminarDepartamento")
            {
                string descDepartamento = this.dtgDepartamentos.Rows[e.RowIndex].Cells["DescDepartamento"].Value.ToString();

                DialogResult result = MessageBox.Show("Si elimina el departamento  \"" + descDepartamento + "\" los usuarios del mismo quedarán sin asignar. \n ¿Desea continuar?", "Eliminar", MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
                if (result.Equals(DialogResult.OK))
                {
                    int idDepartamento = (int)this.dtgDepartamentos.Rows[e.RowIndex].Cells["IdDepartamento"].Value;
                    try
                    {
                        servicioGG.EliminarDepartamento(idDepartamento);
                        ActualizarGestionDepartamentos();
                        MessageBox.Show("Departamento eliminado correctamente.", "Departamentos");
                    }
                    catch (Exception ex)
                    {
                        this.MostrarError("Se ha producido un error:" + ex.Message);
                    }
                }
            }
        }

        private void dtgDepartamentos_CellPainting(object sender, DataGridViewCellPaintingEventArgs e)
        {
            try
            {
                if (e.ColumnIndex >= 0 && this.dtgDepartamentos.Columns[e.ColumnIndex].Name == "EliminarDepartamento" && e.RowIndex >= 0)
                {
                    e.Paint(e.CellBounds, DataGridViewPaintParts.All);
                    DataGridViewButtonCell celBoton = this.dtgDepartamentos.Rows[e.RowIndex].Cells["EliminarDepartamento"] as DataGridViewButtonCell;
                    Image imgDelete = Properties.Resources.No_check;
                    e.Graphics.DrawImage(imgDelete, e.CellBounds.Left + 3, e.CellBounds.Top + 3, 20, 20);
                    this.dtgDepartamentos.Rows[e.RowIndex].Height = 30;
                    this.dtgDepartamentos.Columns[e.ColumnIndex].Width = 30;
                    e.Handled = true;
                }
            }
            catch { }
        }
        #endregion Departamentos

        #region Categorias
        private void ActualizarGestionCategorias()
        {
            try
            {
                List<CategoriaProfesionalDTO> listaCategorias = servicioGG.ObtenerCategorias();
                dtgCategorias.DataSource = listaCategorias;
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error:" + ex.Message);
            }

            pnlEdicionCategoria.Visible = false;
        }

        private void dtgCategorias_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                DataGridViewRow filaSeleccionada = dtgCategorias.Rows[e.RowIndex];
                idCategoriaProfesionalSelected = (int)filaSeleccionada.Cells["IdCategoriaProfesional"].Value;

                CargarPanelEdicionCategoria(idCategoriaProfesionalSelected);
            }
        }

        private void CargarPanelEdicionCategoria(int? idCategoriaProfesionalSelected)
        {
            pnlEdicionCategoria.Visible = true;
            lblDescripcionCategoriaError.Visible = false;

            if (idCategoriaProfesionalSelected.HasValue)
            {
                try
                {
                    CategoriaProfesionalDTO categoria = servicioGG.ObtenerCategoria(idCategoriaProfesionalSelected.Value);
                    if (categoria != null)
                    {
                        txtDescripcionCategoria.Text = categoria.DescCategoriaProfesional;
                    }
                }
                catch (Exception ex)
                {
                    this.MostrarError("Se ha producido un error:" + ex.Message);
                }
            }
            else
                LimpiarPanelEdicionCategoria();

            //Deshabilitar el resto
            pnlGastoTipo.Enabled = false;
            pnlDepartamentos.Enabled = false;
        }

        private void LimpiarPanelEdicionCategoria()
        {
            txtDescripcionCategoria.Text = string.Empty;

            //Habilitar el resto
            pnlGastoTipo.Enabled = true;
            pnlDepartamentos.Enabled = true;
        }

        private void btnNuevaCategoria_Click(object sender, EventArgs e)
        {
            idCategoriaProfesionalSelected = null;
            CargarPanelEdicionCategoria(idCategoriaProfesionalSelected);
        }

        private void btnCancelarCategoria_Click(object sender, EventArgs e)
        {
            pnlEdicionCategoria.Visible = false;
            LimpiarPanelEdicionCategoria();
        }

        private void btnAceptarCategoria_Click(object sender, EventArgs e)
        {
            CategoriaProfesionalDTO categoria = new CategoriaProfesionalDTO();
            categoria.DescCategoriaProfesional = txtDescripcionCategoria.Text;
            categoria.IdCategoriaProfesional = idCategoriaProfesionalSelected.GetValueOrDefault();

            if (ValidarDatosCategoria(categoria))
            {
                try
                {
                    servicioGG.ActualizarCategoria(categoria);
                    ActualizarGestionCategorias();
                    MessageBox.Show("Categorías profesionales actualizadas correctamente.", "Categorías profesionales");
                }
                catch (Exception ex)
                {
                    this.MostrarError("Se ha producido un error:" + ex.Message);
                }
            }
            else
            {
                this.MostrarInformacion("No se han informado los datos correctamente.");
            }
        }

        private bool ValidarDatosCategoria(CategoriaProfesionalDTO categoria)
        {
            bool validado = true;

            lblDescripcionCategoriaError.Visible = string.IsNullOrEmpty(categoria.DescCategoriaProfesional);
            if (lblDescripcionCategoriaError.Visible)
                validado = false;

            return validado;
        }

        private void dtgCategorias_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex >= 0 && this.dtgCategorias.Columns[e.ColumnIndex].Name == "EliminarCategoria")
            {
                string descCategoria = this.dtgCategorias.Rows[e.RowIndex].Cells["DescCategoriaProfesional"].Value.ToString();

                DialogResult result = MessageBox.Show("Si elimina la categoría \"" + descCategoria + "\" los usuarios del mismo quedarán sin asignar. \n ¿Desea continuar?", "Eliminar", MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
                if (result.Equals(DialogResult.OK))
                {
                    int idCategoria = (int)this.dtgCategorias.Rows[e.RowIndex].Cells["IdCategoriaProfesional"].Value;
                    try
                    {
                        servicioGG.EliminarCategoria(idCategoria);
                        ActualizarGestionCategorias();
                        MessageBox.Show("Categoría eliminada correctamente.", "Categorías profesionales");
                    }
                    catch (Exception ex)
                    {
                        this.MostrarError("Se ha producido un error:" + ex.Message);
                    }
                }
            }
        }

        private void dtgCategorias_CellPainting(object sender, DataGridViewCellPaintingEventArgs e)
        {
            try
            {
                if (e.ColumnIndex >= 0 && this.dtgCategorias.Columns[e.ColumnIndex].Name == "EliminarCategoria" && e.RowIndex >= 0)
                {
                    e.Paint(e.CellBounds, DataGridViewPaintParts.All);
                    DataGridViewButtonCell celBoton = this.dtgCategorias.Rows[e.RowIndex].Cells["EliminarCategoria"] as DataGridViewButtonCell;
                    Image imgDelete = Properties.Resources.No_check;
                    e.Graphics.DrawImage(imgDelete, e.CellBounds.Left + 3, e.CellBounds.Top + 3, 20, 20);
                    this.dtgCategorias.Rows[e.RowIndex].Height = 30;
                    this.dtgCategorias.Columns[e.ColumnIndex].Width = 30;
                    e.Handled = true;
                }
            }
            catch { }
        }
        #endregion Categorias

        private void btnSalir_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
