﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using GestorGastos.Administracion.GestorGastosService;

namespace GestorGastos.Administracion
{
    public partial class FormUsuarioEdit : FormBase
    {
        GestorGastosServiceClient servicioGG = new GestorGastosServiceClient();

        int idUsuarioConectado;
        int? idUsuarioSelected;

        public FormUsuarioEdit(int? idUsuario, int idUsuarioAdmin)
        {
            InitializeComponent();
            idUsuarioSelected = idUsuario;
            idUsuarioConectado = idUsuarioAdmin;

            btnBaja.Visible = idUsuarioSelected.HasValue;
            txtNIF.Enabled = !idUsuarioSelected.HasValue;
            txtNIF.ReadOnly = idUsuarioSelected.HasValue;

            //Cargar los combos
            CargarCategorias();
            CargarDepartamentos();
            CargarRoles();
            CargarSupervisores();

            ActivarComboSupervisores();
        }

        private void CargarCategorias()
        {
            try
            {
                List<CategoriaProfesionalDTO> lstCategorias = servicioGG.ObtenerCategorias();

                if (lstCategorias == null)
                    lstCategorias = new List<CategoriaProfesionalDTO>();

                CategoriaProfesionalDTO seleccionar = new CategoriaProfesionalDTO();
                seleccionar.IdCategoriaProfesional = 0;
                seleccionar.DescCategoriaProfesional = "- Seleccionar -";
                lstCategorias.Insert(0, seleccionar);
                cmbCategoria.DataSource = lstCategorias;
                cmbCategoria.DisplayMember = "DescCategoriaProfesional";
                cmbCategoria.ValueMember = "IdCategoriaProfesional";
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error:" + ex.Message);
            }
        }

        private void CargarDepartamentos()
        {
            try
            {
                List<DepartamentoDTO> lstDepartamentos = servicioGG.ObtenerDepartamentos();

                if (lstDepartamentos == null)
                    lstDepartamentos = new List<DepartamentoDTO>();

                DepartamentoDTO seleccionar = new DepartamentoDTO();
                seleccionar.IdDepartamento = 0;
                seleccionar.DescDepartamento = "- Seleccionar -";
                lstDepartamentos.Insert(0, seleccionar);
                cmbDepartamento.DataSource = lstDepartamentos;
                cmbDepartamento.DisplayMember = "DescDepartamento";
                cmbDepartamento.ValueMember = "IdDepartamento";
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error:" + ex.Message);
            }
        }

        private void CargarRoles()
        {
            try
            {
                List<RolDTO> lstRoles = servicioGG.ObtenerRoles();

                if (lstRoles == null)
                    lstRoles = new List<RolDTO>();

                RolDTO seleccionar = new RolDTO();
                seleccionar.IdRol = 0;
                seleccionar.DescRol = "- Seleccionar -";
                lstRoles.Insert(0, seleccionar);
                cmbPerfil.DataSource = lstRoles;
                cmbPerfil.DisplayMember = "DescRol";
                cmbPerfil.ValueMember = "IdRol";
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error:" + ex.Message);
            }
        }

        private void CargarSupervisores()
        {
            try
            {
                List<UsuarioDTO> lstSupervisores = servicioGG.ObtenerUsuariosSupervisores();
                if (lstSupervisores == null)
                    lstSupervisores = new List<UsuarioDTO>();
                
                //si se está editando profesional no tiene que aparecer como supervisor
                if (idUsuarioSelected.HasValue)
                    lstSupervisores.RemoveAll(u => u.IdUsuario == idUsuarioSelected.Value);

                UsuarioDTO seleccionar = new UsuarioDTO();
                seleccionar.IdUsuario = 0;
                seleccionar.NombreCompleto = "- Seleccionar -";
                lstSupervisores.Insert(0, seleccionar);
                cmbUsuarioSuperior.DataSource = lstSupervisores;
                cmbUsuarioSuperior.DisplayMember = "NombreCompleto";
                cmbUsuarioSuperior.ValueMember = "IdUsuario";
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error:" + ex.Message);
            }
        }

        private void FormUsuarioEdit_Load(object sender, EventArgs e)
        {
            lblNIFError.Visible = false;
            lblNombreError.Visible = false;
            lblApellido1Error.Visible = false;
            lblEmailError.Visible = false;
            lblNombreUsuarioError.Visible = false;
            lblContrasenaError.Visible = false;

            if (idUsuarioSelected.HasValue)
            {
                //cargar usuario
                try
                {
                    UsuarioDTO usuario = servicioGG.ObtenerUsuario(idUsuarioSelected.Value);
                    if (usuario != null)
                    {
                        txtNIF.Text = usuario.NIF;
                        txtNombre.Text = usuario.Nombre;
                        txtApellido1.Text = usuario.Apellido1;
                        txtApellido2.Text = usuario.Apellido2;
                        txtEmail.Text = usuario.Email;
                        txtNombreUsuario.Text = usuario.Username;
                        txtContrasena.Text = usuario.Password;
                        cmbCategoria.SelectedValue = usuario.IdCategoriaProfesional.GetValueOrDefault();
                        cmbDepartamento.SelectedValue = usuario.IdDepartamento.GetValueOrDefault();
                        cmbPerfil.SelectedValue = usuario.IdRol.GetValueOrDefault();
                        if (usuario.IdUsuarioSupervisor.HasValue)
                            cmbUsuarioSuperior.SelectedValue = usuario.IdUsuarioSupervisor.GetValueOrDefault();

                        ActivarComboSupervisores();

                        //se podrá eliminar siempre que no sea el mismo usuario administrador (se asegura no eliminar todos los administradores)
                        btnBaja.Visible = idUsuarioSelected.HasValue && idUsuarioSelected.GetValueOrDefault() != idUsuarioConectado;

                        if (usuario.IndBaja.GetValueOrDefault())
                            ActualizarPantallaBaja();
                    }
                }
                catch (Exception ex)
                {
                    this.MostrarError("Se ha producido un error:" + ex.Message);
                }
            }
        }

        private void cmbPerfil_SelectedIndexChanged(object sender, EventArgs e)
        {
            ActivarComboSupervisores();
        }

        private void ActivarComboSupervisores()
        {
            if (cmbPerfil.SelectedIndex > 0)
            {
                int idRol = ((Rol)cmbPerfil.SelectedItem).IdRol;
                if (idRol == (int)RolEnum.EMPLEADO || idRol == (int)RolEnum.SUPERVISOR)
                    cmbUsuarioSuperior.Enabled = true;
                else
                {
                    cmbUsuarioSuperior.Enabled = false;
                    cmbUsuarioSuperior.SelectedIndex = 0;
                }
            }
        }

        private void btnAceptar_Click(object sender, EventArgs e)
        {
            try
            {
                UsuarioDTO usuario = new UsuarioDTO();

                if (idUsuarioSelected.HasValue)
                    usuario = servicioGG.ObtenerUsuario(idUsuarioSelected.Value);

                //si el usuario deja de ser supervisor hay que mostrar aviso (los supervisados quedarán sin asignar, las hojas de éstos las aprobará el responsable de nómina)
                if (usuario != null && usuario.IdRol.GetValueOrDefault() == (int)RolEnum.SUPERVISOR)
                {
                    if (cmbPerfil.SelectedIndex > 0 && ((Rol)cmbPerfil.SelectedItem).IdRol != usuario.IdRol.GetValueOrDefault())
                    {
                        DialogResult result = MessageBox.Show("Si el usuario deja de ser supervisor y tiene supervisados quedarán sin asignar.\n¿Desea continuar?", "Usuario", MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
                        if (result.Equals(DialogResult.Cancel))
                            return;
                    }
                }

                usuario.NIF = txtNIF.Text.ToUpper().Trim();
                usuario.Nombre = txtNombre.Text.ToUpper().Trim();
                usuario.Apellido1 = txtApellido1.Text.ToUpper().Trim();
                usuario.Apellido2 = txtApellido2.Text.ToUpper().Trim();
                usuario.NombreCompleto = (usuario.Nombre + " " + usuario.Apellido1 + " " + usuario.Apellido2).ToString().Trim();
                usuario.Email = txtEmail.Text.Trim();
                usuario.Username = txtNombreUsuario.Text.Trim();
                usuario.Password = txtContrasena.Text.Trim();
                if (cmbCategoria.SelectedIndex > 0)
                    usuario.IdCategoriaProfesional = ((CategoriaProfesional)cmbCategoria.SelectedItem).IdCategoriaProfesional;
                else
                    usuario.IdCategoriaProfesional = null;
                if (cmbDepartamento.SelectedIndex > 0)
                    usuario.IdDepartamento = ((Departamento)cmbDepartamento.SelectedItem).IdDepartamento;
                else
                    usuario.IdDepartamento = null;
                
                if (cmbPerfil.SelectedIndex > 0)
                    usuario.IdRol = ((Rol)cmbPerfil.SelectedItem).IdRol;
                else
                    usuario.IdRol = null;
                if (cmbUsuarioSuperior.SelectedIndex > 0)
                    usuario.UsuarioSupervisor = (Usuario)cmbUsuarioSuperior.SelectedItem;
                else
                    usuario.UsuarioSupervisor = null;

                if (ValidarDatosUsuario(usuario))
                {
                    if (!idUsuarioSelected.HasValue)
                    {
                        string mensajeValidacion = string.Empty;
                        if (EsNifNuevoApto(usuario.NIF, out mensajeValidacion))
                        {
                            if (!string.IsNullOrEmpty(mensajeValidacion))
                            {
                                DialogResult result = MessageBox.Show(mensajeValidacion, "Alta", MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
                                if (result.Equals(DialogResult.OK))
                                {
                                    servicioGG.RehabilitarUsuario(usuario);
                                    this.MostrarInformacion("El usuario se ha actualizado correctamente.");
                                    this.Close();
                                }
                            }
                            else
                            {
                                servicioGG.InsertarUsuario(usuario);
                                this.MostrarInformacion("El usuario se ha actualizado correctamente.");
                                this.Close();
                            }
                        }
                        else
                            this.MostrarInformacion(mensajeValidacion);
                    }
                    else
                    {
                        servicioGG.ActualizarUsuario(usuario);
                        this.MostrarInformacion("El usuario se ha actualizado correctamente.");
                        this.Close();
                    }
                }
                else
                    this.MostrarInformacion("No se han informado los datos correctamente.");
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error:" + ex.Message);
            }
        }

        private bool ValidarDatosUsuario(UsuarioDTO usuario)
        {
            bool validado = true;
            try
            {
                lblNIFError.Visible = string.IsNullOrEmpty(usuario.NIF) || !Common.EsCorrectoNIF(usuario.NIF);
                if (lblNIFError.Visible)
                    validado = false;

                lblNombreError.Visible = string.IsNullOrEmpty(usuario.Nombre);
                if (lblNombreError.Visible)
                    validado = false;

                lblApellido1Error.Visible = string.IsNullOrEmpty(usuario.Apellido1);
                if (lblApellido1Error.Visible)
                    validado = false;

                lblEmailError.Visible = string.IsNullOrEmpty(usuario.Email) || !Common.ValidarEmail(usuario.Email);
                if (lblEmailError.Visible)
                    validado = false;

                if (string.IsNullOrEmpty(usuario.Username) || !Common.ValidarNombreUsuario(usuario.Username))
                {
                    toolTipError.SetToolTip(lblNombreUsuarioError, "Valor incorrecto. Máximo 10 dígitos.");
                    lblNombreUsuarioError.Visible = true;
                    validado = false;
                }
                else
                {
                    try
                    {
                        //Comprobar que no existe el username
                        UsuarioDTO usuarioUsername = servicioGG.ObtenerUsuarioPorUsername(usuario.Username);
                        if (usuarioUsername != null && usuario.IdUsuario > 0 && usuarioUsername.IdUsuario != usuario.IdUsuario)
                        {
                            toolTipError.SetToolTip(lblNombreUsuarioError, "El nombre de usuario ya está registrado.");
                            lblNombreUsuarioError.Visible = true;
                            validado = false;
                        }
                        else if (usuarioUsername != null && usuario.IdUsuario == 0 && !usuarioUsername.IndBaja.GetValueOrDefault()) //se está creando nuevo usuario con NIF existente de baja
                        {
                            toolTipError.SetToolTip(lblNombreUsuarioError, "El nombre de usuario ya está registrado.");
                            lblNombreUsuarioError.Visible = true;
                            validado = false;
                        }
                        else
                            lblNombreUsuarioError.Visible = false;
                    }
                    catch (Exception ex)
                    {
                        this.MostrarError("Se ha producido un error:" + ex.Message);
                    }
                }

                lblContrasenaError.Visible = string.IsNullOrEmpty(usuario.Password) || !Common.ValidarContrasena(usuario.Password.Trim());
                if (lblContrasenaError.Visible)
                    validado = false;

                lblPerfilError.Visible = !usuario.IdRol.HasValue;
                if (lblPerfilError.Visible)
                    validado = false;
                
                if (usuario.IdRol.GetValueOrDefault() == (int)RolEnum.EMPLEADO)
                {
                    lblSupervisorError.Visible = (usuario.UsuarioSupervisor == null);
                    if (lblSupervisorError.Visible)
                        validado = false;
                }
                else
                    lblSupervisorError.Visible = false;
            }
            catch (Exception ex)
            {
                this.MostrarError("Se ha producido un error:" + ex.Message);
            }

            return validado;
        }

        /// <summary>
        /// si se está creando un usuario nuevo, validar que el NIF no exista y si existe mostrar mensaje de confirmación
        /// </summary>
        /// <param name="nif"></param>
        /// <param name="descValidacion"></param>
        /// <returns></returns>
        private bool EsNifNuevoApto(string nif, out string descValidacion)
        {
            descValidacion = string.Empty;

            try
            {
                List<UsuarioDTO> listUsuarios = servicioGG.ObtenerUsuariosPorNIF(nif);
                if (listUsuarios.Count > 0)
                {
                    if (listUsuarios[0].IndBaja == false)
                    {
                        descValidacion = "Ya existe un usuario con el NIF introducido.";
                        return false;
                    }
                    else
                    {
                        descValidacion = "El usuario está de baja. ¿Desea rehabilitarlo?";
                        return true;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Se ha producido un error: \n" + ex.Message, "GestorGastos");
            }
            return true;
        }

        private void btnBaja_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("¿Desea dar de baja al usuario? \nSi tiene supervisados quedarán sin asignar.", "Baja", MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
            if (result.Equals(DialogResult.OK))
            {
                try
                {
                    if (idUsuarioSelected.HasValue)
                        servicioGG.EliminarUsuario(idUsuarioSelected.Value); //Baja lógica del usuario y de sus hojas
                    ActualizarPantallaBaja();
                    this.MostrarInformacion("Usuario ha sido dado de baja.");
                }
                catch (Exception ex)
                {
                    this.MostrarError("Se ha producido un error:" + ex.Message);
                }
            }
        }

        private void btnCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void ActualizarPantallaBaja()
        {
            btnAceptar.Visible = false;
            btnBaja.Visible = false;
            pnlDatosPersonales.Enabled = false;
            pnlDatosProfesionales.Enabled = false;
        }
    }
}

