﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using GestorGastos.App.UserControls;
using GestorGastos.App.GestorGastosService;

namespace GestorGastos.App
{
    public partial class FormPrincipal : Form
    {
        UsuarioDTO usuarioConectado;

        public FormPrincipal(UsuarioDTO usuario)
        {
            InitializeComponent();
            this.StartPosition = FormStartPosition.CenterScreen;
            this.Icon = Properties.Resources.money_safe1;
 
            usuarioConectado = usuario;

            ucInfoUsuarioMenu.Salir += ucInfoUsuarioMenu_Salir;
            ucMisHojas.VerHoja += ucMisHojas_VerHoja;
            ucVerHoja.MisHojas += ucVerHoja_MisHojas;
            ucVerHoja.NuevoGasto += ucVerHoja_NuevoGasto;
            ucNuevoGasto.VolverMisHojas += ucNuevoGasto_VolverMisHojas;
            ucNuevoGasto.VolverVerHoja += ucNuevoGasto_VolverVerHoja;
            ucHojaSupervisados.VerHojaSupervisar += ucHojaSupervisados_VerHojaSupervisar;
            ucVerHoja.HojasSupervisados += ucVerHoja_HojasSupervisados;

            ucInfoUsuarioMenu.CargarDatos(usuario);
            CargarBotonera(usuario);

            if (usuario.IdRol != (int)RolEnum.RESPONSABLE_NOMINA)
            {
                MostrarControl("ucMisHojas");
                ucMisHojas.CargarDatos(usuario);
            }
            else
            {
                MostrarControl("ucHojaSupervisados");
                ucHojaSupervisados.CargarDatos(usuario);
            }
        }

        private void CargarBotonera(UsuarioDTO usuario)
        {
            switch (usuario.IdRol)
            {
                case (int)RolEnum.EMPLEADO:
                    btnMisHojas.Visible = true;
                    btnNuevoGasto.Visible = true;
                    btnHojasSupervisados.Visible = false;
                    break;
                case (int)RolEnum.RESPONSABLE_NOMINA:
                    btnMisHojas.Visible = false;
                    btnNuevoGasto.Visible = false;
                    btnHojasSupervisados.Visible = true;
                    btnHojasSupervisados.Top = 0;
                    break;
                case (int)RolEnum.SUPERVISOR:
                    btnMisHojas.Visible = true;
                    btnNuevoGasto.Visible = true;
                    btnHojasSupervisados.Visible = true;
                    break;
            }
        }

        #region Botones
        private void btnMisHojas_Click(object sender, EventArgs e)
        {
            MostrarControl("ucMisHojas");
            ucMisHojas.CargarDatos(usuarioConectado);
        }

        private void btnNuevoGasto_Click(object sender, EventArgs e)
        {
            MostrarControl("ucNuevoGasto");
            ucNuevoGasto.CargarDatos(null, null, usuarioConectado);
        }

        private void btnHojasSupervisados_Click(object sender, EventArgs e)
        {
            MostrarControl("ucHojaSupervisados");
            ucHojaSupervisados.CargarDatos(usuarioConectado);
        }
        #endregion Botones

        void ucNuevoGasto_VolverMisHojas()
        {
            MostrarControl("ucMisHojas");
            ucMisHojas.CargarDatos(usuarioConectado);
        }

        void ucNuevoGasto_VolverVerHoja(int idHoja)
        {
            MostrarControl("ucVerHoja");
            ucVerHoja.CargarDatos(idHoja, usuarioConectado);
        }

        void ucHojaSupervisados_VerHojaSupervisar(int idHoja, UsuarioDTO usuario)
        {
            MostrarControl("ucVerHoja");
            ucVerHoja.CargarDatos(idHoja, usuarioConectado);
        }

        void ucVerHoja_HojasSupervisados()
        {
            MostrarControl("ucHojaSupervisados");
            ucHojaSupervisados.CargarDatos(usuarioConectado);
        }

        private void MostrarControl(string controlName)
        {
            ucMisHojas.Visible = (controlName == ucMisHojas.Name);
            ucNuevoGasto.Visible = (controlName == ucNuevoGasto.Name);
            ucVerHoja.Visible = (controlName == ucVerHoja.Name);
            ucHojaSupervisados.Visible = (controlName == ucHojaSupervisados.Name);
        }

        void ucMisHojas_VerHoja(int idHoja, UsuarioDTO usuario)
        {
            MostrarControl("ucVerHoja");
            ucVerHoja.CargarDatos(idHoja, usuarioConectado);
        }

        void ucVerHoja_MisHojas()
        {
            MostrarControl("ucMisHojas");
            ucMisHojas.CargarDatos(usuarioConectado);
        }

        void ucVerHoja_NuevoGasto(int? idGasto, int? idHoja, UsuarioDTO usuario)
        {
            MostrarControl("ucNuevoGasto");
            ucNuevoGasto.CargarDatos(idGasto, idHoja, usuario);
        }

        void ucInfoUsuarioMenu_Salir()
        {
            this.Close();
        }
    }
}
