﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using GestorGastos.App.GestorGastosService;

namespace GestorGastos.App.UserControls
{
    public partial class HojaSupervisadosUC : UserControl
    {
        public event VerHojaEventHandler VerHojaSupervisar;

        GestorGastosServiceClient servicioGG = new GestorGastosServiceClient();

        UsuarioDTO usuarioConectado;

        public HojaSupervisadosUC()
        {
            InitializeComponent();

            gridHojas.AutoGenerateColumns = false;

            CargarComboHojasEstado();
        }

        public void CargarDatos(GestorGastosService.UsuarioDTO usuario)
        {
            usuarioConectado = usuario;

            txtNombre.Text = string.Empty;
            txtNif.Text = string.Empty;
            cboEstado.SelectedIndex = 0;
            gridHojas.DataSource = null;
        }

        private void CargarComboHojasEstado()
        {
            try
            {
                List<HojaEstadoTipoDTO> lstEstados = new List<HojaEstadoTipoDTO>();
                HojaEstadoTipoDTO estadoTodos = new HojaEstadoTipoDTO();
                estadoTodos.DescHojaEstadoTipo = "- Todas -";
                estadoTodos.IdHojaEstadoTipo = 0;
                lstEstados.Add(estadoTodos);
                lstEstados.AddRange(servicioGG.ObtenerHojaEstadoTipos());
                cboEstado.DataSource = lstEstados;
                cboEstado.DisplayMember = "DescHojaEstadoTipo";
                cboEstado.ValueMember = "IdHojaEstadoTipo";
                cboEstado.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.InnerException.Message, true).ShowDialog();
            }
        }

        private void btnBuscar_Click(object sender, EventArgs e)
        {
            try
            {
                List<HojaDTO> lstHojas = new List<HojaDTO>();
                int? estado = ((HojaEstadoTipoDTO)cboEstado.SelectedItem).IdHojaEstadoTipo;
                if (estado == 0) estado = null;

                if (usuarioConectado.IdRol == (int)RolEnum.SUPERVISOR)
                    lstHojas = servicioGG.ObtenerHojasASupervisar(usuarioConectado.IdUsuario, estado, txtNif.Text, txtNombre.Text);
                else if (usuarioConectado.IdRol == (int)RolEnum.RESPONSABLE_NOMINA)
                    lstHojas = servicioGG.ObtenerHojasASupervisar(null, estado, txtNif.Text, txtNombre.Text);

                gridHojas.DataSource = lstHojas;
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
        }

        private void gridHojas_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                DataGridViewRow filaSeleccionada = gridHojas.Rows[e.RowIndex];
                int idHojaSelected = (int)filaSeleccionada.Cells["IdHoja"].Value;

                VerHojaSupervisar(idHojaSelected, usuarioConectado);
            }
        }
    }
}
