﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using GestorGastos.App.GestorGastosService;

namespace GestorGastos.App.UserControls
{
    public partial class InfoHojaUC : UserControl
    {
        GestorGastosServiceClient servicioGG = new GestorGastosServiceClient();

        public InfoHojaUC()
        {
            InitializeComponent();
        }

        public int? CargarDatos(int? idHoja, UsuarioDTO usuario)
        {
            try
            {
                HojaDTO hoja = null;
                if (idHoja.HasValue)
                    hoja = servicioGG.ObtenerHoja(idHoja.Value);
                else
                {
                    //En el caso de añadir nuevo gasto, obtener si existe una hoja vigente
                    hoja = servicioGG.ObtenerHojaVigente(usuario.IdUsuario);
                    if (hoja != null)
                        idHoja = hoja.IdHoja;
                }

                if (hoja == null)
                {
                    hoja = new HojaDTO();
                    if (usuario != null)
                        hoja.Referencia = DateTime.Today.Month + "/" + DateTime.Today.Year;
                    hoja.NombreUsuarioAprobador = usuario.NombreUsuarioSupervisor;
                    hoja.Estado = "Abierta";
                    hoja.FechaCreacion = DateTime.Now;
                }

                if(!hoja.IdUsuarioCreador.HasValue)
                      lblTituloHoja.Text = hoja.Referencia + " - " + usuario.NombreCompleto.ToUpper(); //hoja nueva
                else if(hoja.IdUsuarioCreador.GetValueOrDefault() == usuario.IdUsuario)
                    lblTituloHoja.Text = hoja.Referencia + " - " + usuario.NombreCompleto.ToUpper(); //hoja del usuario conectado
                else
                    lblTituloHoja.Text = hoja.Referencia + " - " + hoja.NombreUsuarioCreador.ToUpper(); //hoja de otro usuario 

                lblEstado.Text = hoja.Estado;
                if (!string.IsNullOrEmpty(hoja.NombreUsuarioAprobador))
                    lblSupervisor.Text = hoja.NombreUsuarioAprobador;
                else
                    lblSupervisor.Text = "---";
                if (hoja.FechaCreacion.HasValue)
                    lblFechaCreacion.Text = hoja.FechaCreacion.Value.ToShortDateString();
                if (hoja.FechaSupervision.HasValue)
                    lblFechaSupervision.Text = hoja.FechaSupervision.Value.ToShortDateString();
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
            return idHoja;
        }
    }
}
