﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using GestorGastos.App.GestorGastosService;

namespace GestorGastos.App.UserControls
{
    public delegate void VerHojaEventHandler(int idHoja, UsuarioDTO usuario);

    public partial class MisHojasUC : UserControl
    {
        public event VerHojaEventHandler VerHoja;

        GestorGastosServiceClient servicioGG = new GestorGastosServiceClient();

        UsuarioDTO usuarioConectado;

        public MisHojasUC()
        {
            InitializeComponent();

            gridHojas.AutoGenerateColumns = false;
        }

        public void CargarDatos(GestorGastosService.UsuarioDTO usuario)
        {
            usuarioConectado = usuario;

            CargarComboHojasEstado();
            CargarMisHojas();
        }

        private void CargarMisHojas()
        {
            try
            {
                List<HojaDTO> lstHojas = new List<HojaDTO>();
                int estado = ((HojaEstadoTipoDTO)cboEstado.SelectedItem).IdHojaEstadoTipo;

                if (estado == 0)
                    lstHojas = servicioGG.ObtenerHojasUsuario(usuarioConectado.IdUsuario);
                else
                    lstHojas = servicioGG.ObtenerHojasUsuarioByEstado(usuarioConectado.IdUsuario, estado);

                gridHojas.DataSource = lstHojas;
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
        }

        private void CargarComboHojasEstado()
        {
            try
            {
                List<HojaEstadoTipoDTO> lstEstados = new List<HojaEstadoTipoDTO>();
                HojaEstadoTipoDTO estadoTodos = new HojaEstadoTipoDTO();
                estadoTodos.DescHojaEstadoTipo = "- Todas -";
                estadoTodos.IdHojaEstadoTipo = 0;
                lstEstados.Add(estadoTodos);
                lstEstados.AddRange(servicioGG.ObtenerHojaEstadoTipos());
                cboEstado.DataSource = lstEstados;
                cboEstado.DisplayMember = "DescHojaEstadoTipo";
                cboEstado.ValueMember = "IdHojaEstadoTipo";
                cboEstado.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
        }

        private void cboEstado_SelectedIndexChanged(object sender, EventArgs e)
        {
            CargarMisHojas();
        }

        private void gridHojas_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                DataGridViewRow filaSeleccionada = gridHojas.Rows[e.RowIndex];
                int idHojaSelected = (int)filaSeleccionada.Cells["IdHoja"].Value;

                VerHoja(idHojaSelected, usuarioConectado);
            }
        }
    }
}
