﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using GestorGastos.App.GestorGastosService;
using System.IO;

namespace GestorGastos.App.UserControls
{
    public delegate void Volver_MisHojasEventHandler();
    public delegate void Volver_VerHojaEventHandler(int idHoja);
    public delegate void Volver_HojasSupervisadosEventHandler();

    public partial class NuevoGastoUC : UserControl
    {
        public event Volver_MisHojasEventHandler VolverMisHojas;
        public event Volver_VerHojaEventHandler VolverVerHoja;
        public event Volver_HojasSupervisadosEventHandler VolverHojasSupervisados;

        GestorGastosServiceClient servicioGG = new GestorGastosServiceClient();

        int? idGastoSeleccionado;
        UsuarioDTO usuarioConectado;
        int? idHojaSeleccionada;

        byte[] fichero;

        public NuevoGastoUC()
        {
            InitializeComponent();
        }

        public void CargarDatos(int? idGasto, int? idHoja, UsuarioDTO  usuario)
        {
            idGastoSeleccionado = idGasto;
            usuarioConectado = usuario;

            idHojaSeleccionada = ucInfoHoja.CargarDatos(idHoja, usuario);

            CargarTiposGastos();

            LimpiarControles();

            if (idGasto.HasValue)
                CargarGasto(idGasto.Value); //Editar gasto
            else
            {
                HabilitarEdicionGasto(true);
                btnEliminar.Visible = false;
            }
        }

        private void LimpiarControles()
        {
            cboGastoTipo.SelectedValue = 0;
            dtpFechaGasto.Value = DateTime.Today;
            txtImporte.Text = string.Empty;
            txtCantidad.Text = string.Empty;
            lblImporteTotal.Text = "-";
            fichero = null;
            CargarDatosFichero(fichero);

            lblCantidadError.Visible = false;
            lblFechaGastoError.Visible = false;
            lblFicheroError.Visible = false;
            lblGastoTipoError.Visible = false;
            lblImporteError.Visible = false;
        }

        private void CargarTiposGastos()
        {
            try
            {
                List<GastoTipoDTO> lstGastoTipos = servicioGG.ObtenerGastosTipos();

                if (lstGastoTipos == null)
                    lstGastoTipos = new List<GastoTipoDTO>();

                GastoTipoDTO seleccionar = new GastoTipoDTO();
                seleccionar.IdGastoTipo = 0;
                seleccionar.DescGastoTipo = "- Seleccionar -";
                lstGastoTipos.Insert(0, seleccionar);
                cboGastoTipo.DataSource = lstGastoTipos;
                cboGastoTipo.DisplayMember = "DescGastoTipo";
                cboGastoTipo.ValueMember = "IdGastoTipo";
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
        }

        private void CargarGasto(int idGasto)
        {
            try
            {
                GastoDTO gasto = servicioGG.ObtenerGastoPorUsuario(idGasto, usuarioConectado.IdUsuario);

                if (gasto != null)
                {
                    cboGastoTipo.SelectedValue = gasto.IdGastoTipo.GetValueOrDefault();
                    txtImporte.Text = gasto.ImporteUnidad.GetValueOrDefault().ToString();
                    txtCantidad.Text = gasto.Unidades.ToString();
                    if (gasto.Fecha.HasValue)
                        dtpFechaGasto.Value = gasto.Fecha.Value;
                    lblImporteTotal.Text = gasto.ImporteTotal.GetValueOrDefault().ToString("0.00") + " €";

                    if (gasto.Fichero != null)
                        fichero = gasto.Fichero;

                    CargarDatosFichero(gasto.Fichero);

                    HabilitarEdicionGasto(gasto.PuedeEditarGasto.GetValueOrDefault());
                   
                }
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
        }

        private void HabilitarEdicionGasto(bool permisoEditar)
        {
            cboGastoTipo.Enabled = permisoEditar;
            dtpFechaGasto.Enabled = permisoEditar;
            txtCantidad.Enabled = permisoEditar;
            txtImporte.Enabled = permisoEditar;
            txtFichero.Enabled = permisoEditar;
            btnFichero.Enabled = permisoEditar;
            lnkEliminar.Enabled = permisoEditar;

            btnAceptar.Visible = permisoEditar;
            btnEliminar.Visible = permisoEditar;
        }

        private void CargarDatosFichero(byte[] fichero)
        {
            if (fichero != null)
            {
                lnkEliminar.Visible = true;
                lnkVerJustificante .Visible = true;
                btnFichero.Visible = false;
            }
            else
            {
                lnkEliminar.Visible = false;
                lnkVerJustificante.Visible = false;
                btnFichero.Visible = true;
            }
        }

        private void btnFichero_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofdJustificante = new OpenFileDialog();
            ofdJustificante.Filter = "JPG Files (*.jpg)|*.jpg|JPEG Files (*.jpeg)|*.jpeg|PNG Files (*.png)|*.png|GIF Files (*.gif)|*.gif";

            if (ofdJustificante.ShowDialog() == DialogResult.OK)
            {
                txtFichero.Text = ofdJustificante.FileName;
                fichero = File.ReadAllBytes(txtFichero.Text);
            }

            lnkEliminar.Visible = !string.IsNullOrEmpty(txtFichero.Text);
            lnkVerJustificante.Visible = !string.IsNullOrEmpty(txtFichero.Text);
        }

        private void btnAceptar_Click(object sender, EventArgs e)
        {
            GastoDTO gasto = new GastoDTO();

            if (idGastoSeleccionado.HasValue)
                gasto.IdGasto = idGastoSeleccionado.Value;

            gasto.Fecha = dtpFechaGasto.Value;

            if (!string.IsNullOrEmpty(txtFichero.Text))
                gasto.Fichero = File.ReadAllBytes(txtFichero.Text);

            if (cboGastoTipo.SelectedIndex > 0)
                gasto.IdGastoTipo = ((GastoTipoDTO)cboGastoTipo.SelectedItem).IdGastoTipo;
            else
                gasto.IdGastoTipo = null;

            if (!string.IsNullOrEmpty(txtImporte.Text))
            {
                string importeTexto = txtImporte.Text.Replace('.', ',');
                importeTexto = String.Format("{0:0.00}", importeTexto);
                decimal importe;
                if (decimal.TryParse(importeTexto, out importe))
                    gasto.ImporteUnidad = importe;
            }

            int unidades;
            if (int.TryParse(txtCantidad.Text, out unidades))
                gasto.Unidades = unidades;

            try
            {
                if (ValidarGasto(gasto))
                {
                    servicioGG.ActualizarGasto(gasto, usuarioConectado.IdUsuario);
                    new FormAviso("El gasto se ha actualizado correctamente.", false).ShowDialog();

                    if (idGastoSeleccionado.HasValue)
                        VolverVerHoja(idHojaSeleccionada.Value);  //volver VerHoja
                    else
                        VolverMisHojas(); //volver a MisHojas 
                }
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
        }

        private void btnCancelar_Click(object sender, EventArgs e)
        {
            try
            {
                if (idHojaSeleccionada.HasValue)
                    VolverVerHoja(idHojaSeleccionada.Value);  //volver VerHoja
                else
                    VolverMisHojas();   //volver a MisHojas 
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
        }

        private void btnEliminar_Click(object sender, EventArgs e)
        {
            if (idGastoSeleccionado.HasValue)
            {
                FormConfirmacion formConfirmar = new FormConfirmacion("¿Desea eliminar el gasto?");
                if (formConfirmar.ShowDialog() == DialogResult.OK)
                {
                    try
                    { if (idGastoSeleccionado.HasValue)
                        if (servicioGG.EliminarGasto(idGastoSeleccionado.Value))
                        {
                            new FormAviso("Gasto eliminado correctamente.", false).ShowDialog();
                            VolverVerHoja(idHojaSeleccionada.Value);
                        }
                    }
                    catch (Exception ex)
                    {
                        new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
                    }
                }
            }
        }

        private bool ValidarGasto(GastoDTO gasto)
        {
            bool validado = true;

            lblGastoTipoError.Visible = !gasto.IdGastoTipo.HasValue;
            if (lblGastoTipoError.Visible)
                validado = false;

            //Validar que la fecha no es superior al mes actual, si se podrán insertar gastos de fecha anteriores
            DateTime fechaPlazo = new DateTime(DateTime.Today.Year, DateTime.Today.Month, 1).AddMonths(1);
            lblFechaGastoError.Visible = (gasto.Fecha.GetValueOrDefault() >= fechaPlazo);
            if (lblFechaGastoError.Visible)
                validado = false;

            lblCantidadError.Visible = (gasto.Unidades.GetValueOrDefault() <= 0);
            if (lblCantidadError.Visible)
                validado = false;

            lblImporteError.Visible = (gasto.ImporteUnidad.GetValueOrDefault() <= 0);
            if (lblImporteError.Visible)
                validado = false;

            if (pnlFichero.Visible)
            {
                lblFicheroError.Visible = (gasto.Fichero == null);
                if (lblFicheroError.Visible)
                    validado = false;
            }

            return validado;
        }

        private void lnkEliminar_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            txtFichero.Text = string.Empty;
            lnkEliminar.Visible = false;
            lnkVerJustificante.Visible = false;
            txtFichero.Visible = true;
            btnFichero.Visible = true;

            fichero = null;
        }

        private void cboGastoTipo_SelectedIndexChanged(object sender, EventArgs e)
        {
            txtImporte.Text = string.Empty;
            ActualizarPantallaPorTipoGasto();
        }

        private void dtpFechaGasto_ValueChanged(object sender, EventArgs e)
        {
            ActualizarPantallaPorTipoGasto();
        }

        private void ActualizarPantallaPorTipoGasto()
        {
            try
            {
                //Actualizar visualización de controles según tipo de gasto y la fecha
                GastoTipoDTO gastoTipoSeleccionado = (GastoTipoDTO)cboGastoTipo.SelectedItem;
                DateTime fecha = dtpFechaGasto.Value;

                if (gastoTipoSeleccionado.IndAdjuntarFichero.GetValueOrDefault())
                    pnlFichero.Visible = true;
                else
                {
                    pnlFichero.Visible = false;
                    txtFichero.Text = string.Empty;
                    lblFicheroError.Visible = false;
                }

                if (gastoTipoSeleccionado.IndImporteLibre.GetValueOrDefault())
                    txtImporte.Enabled = true;
                else
                {
                    txtImporte.Enabled = false;
                    if (gastoTipoSeleccionado.IndImporteDiferenteFestivo.GetValueOrDefault() && EsFechaFestiva(fecha))
                        txtImporte.Text = gastoTipoSeleccionado.ImporteFestivo.GetValueOrDefault().ToString();
                    else
                        txtImporte.Text = gastoTipoSeleccionado.Importe.GetValueOrDefault().ToString();
                }
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
        }

        /// <summary>
        /// Comprobar si la fecha introducida está marcada como no laborable en el calendario 
        /// </summary>
        /// <param name="fecha"></param>
        /// <returns></returns>
        private bool EsFechaFestiva(DateTime fecha)
        {
            bool result = false;

            DayOfWeek diaSemana = dtpFechaGasto.Value.DayOfWeek;
            if (diaSemana == DayOfWeek.Saturday || diaSemana == DayOfWeek.Sunday)
                result = true;
            else
            {
                //para el resto de días comprobar el calendario laboral
                try
                {
                    result = servicioGG.EsFechaFestiva(fecha.Date);
                }
                catch (Exception ex)
                {
                    new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
                }
            }
            return result;
        }

        private void lnkVerJustificante_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
             if (fichero != null)
                new FormVerJustificante(fichero).ShowDialog();
        }
    }
}
