﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using GestorGastos.App.GestorGastosService;

namespace GestorGastos.App.UserControls
{
    public delegate void MisHojasEventHandler();
    public delegate void NuevoGastoEventHandler(int? idGasto, int? idHojaSeleccionada, UsuarioDTO usuario);
    public delegate void HojasSupervisadosEventHandler();

    public partial class VerHojaUC : UserControl
    {
        public event MisHojasEventHandler MisHojas;
        public event NuevoGastoEventHandler NuevoGasto;
        public event HojasSupervisadosEventHandler HojasSupervisados;

        GestorGastosServiceClient servicioGG = new GestorGastosServiceClient();

        int idHojaSeleccionada;
        UsuarioDTO usuarioConectado;

        public VerHojaUC()
        {
            InitializeComponent();

            gridGastos.AutoGenerateColumns = false;
        }

        public void CargarDatos(int idHoja, UsuarioDTO usuario)
        {
            try
            {
                idHojaSeleccionada = idHoja;
                usuarioConectado = usuario;

                ucInfoHoja.CargarDatos(idHojaSeleccionada, usuario);

                HojaDTO hojaSeleccionada = servicioGG.ObtenerHojaPorUsuario(idHoja, usuario.IdUsuario);

                if (hojaSeleccionada != null)
                {
                    lblTotal.Text = hojaSeleccionada.Importe.GetValueOrDefault().ToString() + " €";

                    if (hojaSeleccionada.IdHojaEstadoTipo == (int)HojaEstadoTipoEnum.DENEGADA)
                    {
                        lblMotivoDenegacion.Text = "Motivo de la denegación: " + hojaSeleccionada.DesMotivoDenegada;
                        lblMotivoDenegacion.Visible = true;
                    }
                    else
                        lblMotivoDenegacion.Visible = false;

                    btnAprobar.Visible = hojaSeleccionada.PuedeSupervisarHoja.GetValueOrDefault();
                    btnDenegar.Visible = hojaSeleccionada.PuedeSupervisarHoja.GetValueOrDefault();
                    btnEliminar.Visible = hojaSeleccionada.PuedeEliminarHoja.GetValueOrDefault();
                }
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }

            CargarGastosHoja(idHoja);
        }

        private void CargarGastosHoja(int idHoja)
        {
            try
            {
                List<GastoDTO> lstGastos = new List<GastoDTO>();
                lstGastos = servicioGG.ObtenerGastosDeHoja(idHoja);

                gridGastos.DataSource = lstGastos;
            }
            catch (Exception ex)
            {
                new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
            }
        }

        private void btnEliminar_Click(object sender, EventArgs e)
        {
            FormConfirmacion formConfirmar = new FormConfirmacion("¿Desea eliminar la hoja, perderá todos los gastos de la misma?");
            if (formConfirmar.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    if (servicioGG.EliminarHoja(idHojaSeleccionada))
                    {
                        new FormAviso("Hoja eliminada correctamente.", false).ShowDialog();
                        MisHojas();
                    }
                }
                catch (Exception ex)
                {
                    new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
                }
            }
        }

        private void btnAprobar_Click(object sender, EventArgs e)
        {
            FormConfirmacion formConfirmar = new FormConfirmacion("¿Desea aprobar la hoja?");
            if (formConfirmar.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    if (servicioGG.AprobarHoja(idHojaSeleccionada, usuarioConectado.IdUsuario))
                    {
                        new FormAviso("Hoja aprobada correctamente.", false).ShowDialog();
                        HojasSupervisados(); 
                    }
                }
                catch (Exception ex)
                {
                    new FormAviso("ERROR: Contacte con el administrador. \n" + ex.Message, true).ShowDialog();
                }
            }
        }

        private void btnDenegar_Click(object sender, EventArgs e)
        {
            FormDenegacionHoja formDenegacion = new FormDenegacionHoja(idHojaSeleccionada, usuarioConectado);
            if (formDenegacion.ShowDialog() == DialogResult.OK)
                HojasSupervisados();
        }

        private void gridGastos_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                DataGridViewRow filaSeleccionada = gridGastos.Rows[e.RowIndex];
                int idGastoSelected = (int)filaSeleccionada.Cells["IdGasto"].Value;

                NuevoGasto(idGastoSelected, idHojaSeleccionada, usuarioConectado);
            }
        }
    }
}
