﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


using GestorGastos.Data;
using GestorGastos.Model;

namespace GestorGastos.Business
{
    public class GastoBL
    {
        public static List<Gasto> GetGastosByIdHoja(int idHoja)
        {
            List<Gasto> listaGastos = GastoDL.GetGastosByIdHoja(idHoja);

            foreach (Gasto gasto in listaGastos)
            {
                GetDatosGasto(gasto);
            }

            return listaGastos;
        }

        private static Gasto GetDatosGasto(Gasto gasto)
        {
            if (gasto != null)
            {
                if (gasto.IdGastoTipo.HasValue)
                    gasto.GastoTipo = GastoTipoDL.GetGastoTipo(gasto.IdGastoTipo.Value);

                if (gasto.IdHoja.HasValue)
                    gasto.Hoja = HojaDL.GetHoja(gasto.IdHoja.Value);
            }

            return gasto;
        }

        public static Gasto GetGasto(int idGasto)
        {
            Gasto gasto = GastoDL.GetGasto(idGasto);
            return GetDatosGasto(gasto);
        }

        public static void ActualizarGasto(Gasto gasto, int idUsuario)
        {
            int? idHojaGasto = null;

            gasto.ImporteTotal = gasto.Unidades.GetValueOrDefault() * gasto.ImporteUnidad.GetValueOrDefault();
            gasto.IndBaja = false;

            if (gasto.IdGasto > 0)
            {
                Gasto gastoBD = GetGasto(gasto.IdGasto);
                if (gastoBD != null)
                {
                    idHojaGasto = gastoBD.IdHoja;
                    gasto.IdHoja = idHojaGasto;
                    GastoDL.UpdateGasto(gasto);
                }
            }
            else
            {
                //Hoja vigente
                Hoja hojaVigente = HojaBL.GetHojaVigente(idUsuario);

                if (hojaVigente != null)
                {
                    idHojaGasto = hojaVigente.IdHoja;
                    gasto.IdHoja = hojaVigente.IdHoja;
                }
                else
                {
                    idHojaGasto = HojaBL.CrearHoja(idUsuario);
                    if (idHojaGasto.HasValue)
                        gasto.IdHoja = idHojaGasto.Value;
                }
                GastoDL.AddGasto(gasto);
            }

            if (idHojaGasto.HasValue)
                HojaBL.ActualizarImporteTotal(idHojaGasto.Value);
        }

        public static bool PuedeEditarGastoByUsuario(int idGasto, int idUsuario)
        {
            bool result = false;
            Gasto gasto = GetGasto(idGasto);
            if (gasto != null)
            {
                Hoja hoja = HojaBL.GetHoja(gasto.IdHoja.GetValueOrDefault());
                if (hoja != null)
                {
                    if (hoja.IdHojaEstadoTipo == (int)HojaEstadoTipoEnum.ABIERTA)
                    {
                        int idUsuarioHoja = hoja.IdUsuarioCreador.GetValueOrDefault();
                        if (idUsuarioHoja == idUsuario)
                            result = true;
                    }
                }
            }
            return result;
        }

        public static bool EliminarGasto(int idGasto)
        {
            Gasto gasto = GastoDL.GetGasto(idGasto);
            if (gasto != null)
            {
                gasto.IndBaja = true;
                GastoDL.UpdateGasto(gasto);
                return true;
            }
            else
                return false;
        }
    }
}