﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using GestorGastos.Data;
using GestorGastos.Model;

namespace GestorGastos.Business
{
    public class HojaBL
    {
        public static List<Hoja> GetHojasByIdUsuario(int idUsuario)
        {
            List<Hoja> listHojas = HojaDL.GetHojasByIdUsuario(idUsuario);

            foreach (Hoja hoja in listHojas)
                GetDatosHoja(hoja);

            return listHojas;
        }

        private static Hoja GetDatosHoja(Hoja hoja)
        {
            if (hoja != null)
            {
                if (hoja.IdUsuarioCreador.HasValue)
                    hoja.Usuario = UsuarioDL.GetUsuario(hoja.IdUsuarioCreador.Value);

                if (hoja.IdHojaEstadoTipo.HasValue)
                    hoja.HojaEstadoTipo = GetHojaEstadoTipo(hoja.IdHojaEstadoTipo.Value);

                //si la hoja esta aprobada o denegada tendrá IdUsuarioSupervisor sino recuperar el supervisor del usuario creador de la hoja
                if (hoja.IdUsuarioSupervisor.HasValue)
                    hoja.UsuarioSupervisor = UsuarioDL.GetUsuario(hoja.IdUsuarioSupervisor.Value);
                else
                    hoja.UsuarioSupervisor = UsuarioDL.GetUsuarioSupervisor(hoja.IdUsuarioCreador.GetValueOrDefault());
            }

            return hoja;
        }

        public static HojaEstadoTipo GetHojaEstadoTipo(int idHojaEstadoTipo)
        {
            return HojaDL.GetHojaEstadoTipo(idHojaEstadoTipo);
        }

        public static List<HojaEstadoTipo> GetAllHojaEstadoTipo()
        {
            return HojaDL.GetAllHojaEstadoTipo();
        }

        public static List<Hoja> GetHojasByIdUsuarioHojaEstadoTipo(int idUsuario, int idHojaEstadoTipo)
        {
            List<Hoja> listHojas = HojaDL.GetHojasByIdUsuarioHojaEstadoTipo(idUsuario, idHojaEstadoTipo);

            foreach (Hoja hoja in listHojas)
                GetDatosHoja(hoja);

            return listHojas;
        }

        public static Hoja GetHoja(int idHoja)
        {
            Hoja hoja = HojaDL.GetHoja(idHoja);
            if (hoja != null)
                hoja = GetDatosHoja(hoja);

            return hoja;
        }

        public static bool EliminarHoja(int idHoja)
        {
            Hoja hoja = HojaDL.GetHoja(idHoja);
            if (hoja != null)
            {
                hoja.IndBaja = true;
                return HojaDL.UpdateHoja(hoja);
            }
            else
                return false;
        }

        public static bool AprobarHoja(int idHoja, int idUsuarioSupervisor)
        {
            Hoja hoja = HojaDL.GetHoja(idHoja);
            if (hoja != null)
            {
                hoja.IdHojaEstadoTipo = (int)HojaEstadoTipoEnum.APROBADA;
                hoja.IdUsuarioSupervisor = idUsuarioSupervisor;
                hoja.FechaSupervision = DateTime.Today;
                return HojaDL.UpdateHoja(hoja);
            }
            else
                return false;
        }

        public static bool DenegarHoja(int idHoja, string motivo, int idUsuarioSupervisor)
        {
            Hoja hoja = HojaDL.GetHoja(idHoja);
            if (hoja != null)
            {
                hoja.IdHojaEstadoTipo = (int)HojaEstadoTipoEnum.DENEGADA;
                hoja.IdUsuarioSupervisor = idUsuarioSupervisor;
                hoja.DesMotivoDenegada = motivo;
                hoja.FechaSupervision = DateTime.Today;
                return HojaDL.UpdateHoja(hoja);
            }
            else
                return false;
        }

        public static bool PuedeEliminarHojaByUsuario(int idHoja, int idUsuario)
        {
            bool result = false;
            Hoja hoja = HojaBL.GetHoja(idHoja);
            if (hoja != null)
            {
                if (hoja.IdHojaEstadoTipo == (int)HojaEstadoTipoEnum.ABIERTA)
                {
                    int idUsuarioHoja = hoja.IdUsuarioCreador.GetValueOrDefault();
                    if (idUsuarioHoja == idUsuario)
                        result = true;
                }
            }
            return result;
        }

        /// <summary>
        /// Puede supervisar la hoja el Responsable de nómina si el usuario de la hoja no tiene supervisor y el supervisor del usuario de la hoja
        /// </summary>
        /// <param name="idHoja"></param>
        /// <param name="idUsuario"></param>
        /// <returns></returns>
        public static bool PuedeSupervisarHojaByUsuario(int idHoja, int idUsuario)
        {
            bool result = false;
            Hoja hoja = HojaBL.GetHoja(idHoja);
            if (hoja != null)
            {
                if (hoja.IdHojaEstadoTipo == (int)HojaEstadoTipoEnum.ABIERTA)
                {
                    Usuario usuario = UsuarioBL.GetUsuario(idUsuario);
                    Usuario usuarioHoja = UsuarioBL.GetUsuario(hoja.IdUsuarioCreador.GetValueOrDefault());

                    if (usuario != null && usuarioHoja != null)
                    {
                        if (usuarioHoja.UsuarioSupervisor == null)
                        {
                            if (usuario.IdRol == (int)RolEnum.RESPONSABLE_NOMINA)
                                result = true;
                        }
                        else
                        {
                            if (usuario.IdUsuario == usuarioHoja.UsuarioSupervisor.IdUsuario)
                                result = true;
                        }
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// Devuelve el identificador de la hoja vigente según el mes en curso para el usuario indicado
        /// </summary>
        /// <returns></returns>
        public static Hoja GetHojaVigente(int idUsuario)
        {
            Hoja hojaVigente = HojaDL.GetHojaVigente(idUsuario);
            if (hojaVigente != null)
            {
                hojaVigente = GetHoja(hojaVigente.IdHoja);
                return hojaVigente;
            }
            return null;
        }

        public static int? CrearHoja(int idUsuario)
        {
            Hoja hoja = new Hoja();
            hoja.FechaCreacion = DateTime.Now;
            hoja.IdHojaEstadoTipo = (int)HojaEstadoTipoEnum.ABIERTA;
            hoja.IdUsuarioCreador = idUsuario;
            hoja.IndBaja = false;
            hoja.Referencia = DateTime.Today.Month + "/" + DateTime.Today.Year;

            return HojaDL.AddHoja(hoja);
        }

        public static void ActualizarImporteTotal(int idHoja)
        {
            HojaDL.ActualizarImporteTotal(idHoja);
        }

        public static List<Hoja> GetHojasSupervisar(int? idUsuarioSupervisor, int? idHojaEstadoTipo, string nif, string nombre)
        {
            List<Hoja> listHojas = HojaDL.GetHojasSupervisar(idUsuarioSupervisor, idHojaEstadoTipo, nif, nombre);
            
            foreach (Hoja hoja in listHojas)
                GetDatosHoja(hoja);

            return listHojas;
        }

        public static void ActualizarHojasBaja(int idUsuario, bool indBaja)
        {
            HojaDL.UpdateHojasBaja(idUsuario, indBaja);
        }
    }
}
