﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using GestorGastos.Data;
using GestorGastos.Model;

namespace GestorGastos.Business
{
    public class UsuarioBL
    {
        public static List<Usuario> GetUsuariosByNifNombre(string NIF, string nombre)
        {
            List<Usuario> listUsuarios = UsuarioDL.GetUsuariosByNifNombre(NIF, nombre);
            foreach (Usuario usuario in listUsuarios)
                GetDatosUsuario(usuario);
            return listUsuarios;
        }

        public static List<Usuario> GetUsuariosByNif(string NIF)
        {
            List<Usuario> listUsuarios = UsuarioDL.GetUsuariosByNif(NIF);
            foreach (Usuario usuario in listUsuarios)
                GetDatosUsuario(usuario);
            return listUsuarios;
        }

        public static List<Usuario> GetUsuariosByNombre(string nombre)
        {
            List<Usuario> listUsuarios = UsuarioDL.GetUsuariosByNombre(nombre);
            foreach (Usuario usuario in listUsuarios)
                GetDatosUsuario(usuario);
            return listUsuarios;
        }

        private static Usuario GetDatosUsuario(Usuario usuario)
        {
            if (usuario != null)
            {
                if (usuario.IdCategoriaProfesional.HasValue)
                    usuario.CategoriaProfesional = CategoriaProfesionalDL.GetCategoria(usuario.IdCategoriaProfesional.Value);

                if (usuario.IdDepartamento.HasValue)
                    usuario.Departamento = DepartamentoDL.GetDepartamento(usuario.IdDepartamento.Value);

                if (usuario.IdRol.HasValue)
                    usuario.Rol = RolDL.GetRol(usuario.IdRol.Value);

                Usuario usuarioSupervisor = UsuarioDL.GetUsuarioSupervisor(usuario.IdUsuario);
                if (usuarioSupervisor != null)
                    usuario.UsuarioSupervisor = usuarioSupervisor;
            }

            return usuario;
        }

        public static Usuario GetUsuario(int idUsuario)
        {
            Usuario usuario = UsuarioDL.GetUsuario(idUsuario);
            if (usuario != null)
                usuario = GetDatosUsuario(usuario);
            return usuario;
        }

        public static List<Usuario> GetUsuariosSupervisores()
        {
            return UsuarioDL.GetUsuariosSupervisores();
        }

        public static List<Usuario> GetUsuarios()
        {
            return UsuarioDL.GetUsuarios();
        }

        public static void ActualizarUsuario(Usuario usuario)
        {
            usuario.IndBaja = false;
            Usuario usuarioBD = GetUsuario(usuario.IdUsuario);
            if (usuarioBD != null)
            {
                if (usuarioBD.IdRol.GetValueOrDefault() == (int)RolEnum.SUPERVISOR && usuario.IdRol.GetValueOrDefault() != (int)RolEnum.SUPERVISOR)
                    UsuarioDL.DeleteUsuariosSupervisados(usuario.IdUsuario);

                UsuarioDL.UpdateUsuario(usuario);
            }
            else
                throw new Exception("Error: ActualizarUsuario (el usuario no existe).");
        }

        public static void RehabilitarUsuario(Usuario usuario)
        {
            //Recuperar el usuario con el NIF
            List<Usuario> lstUsuarios = GetUsuariosByNif(usuario.NIF);
            if (lstUsuarios != null && lstUsuarios.Count > 0)
            {
                usuario.IdUsuario = lstUsuarios[0].IdUsuario;
                ActualizarUsuario(usuario);
            }
            else
                throw new Exception("Error: RehabilitarUsuario (el usuario no existe).");
        }

        public static void InsertarUsuario(Usuario usuario)
        {
            usuario.IndBaja = false;
            Usuario usuarioBD = GetUsuario(usuario.IdUsuario);
            if (usuarioBD == null)
                UsuarioDL.AddUsuario(usuario);
            else
                throw new Exception("Error: InsertarUsuario (el usuario ya existe).");
        }

        /// <summary>
        /// Baja lógica del usuario
        /// Se dan de baja también sus hojas, ya no tienen que ser supervisadas
        /// Si tiene usuarios supervisados quedan sin supervisor (los aprueba el responsable de nómina) 
        /// </summary>
        /// <param name="idUsuario"></param>
        public static void DeleteUsuario(int idUsuario)
        {
            HojaBL.ActualizarHojasBaja(idUsuario, true);
            UsuarioDL.DeleteUsuariosSupervisados(idUsuario);
            UsuarioDL.DeleteUsuario(idUsuario);
        }

        public static int? IsLoginUsuario(string username, string password)
        {
            if (!string.IsNullOrEmpty(username) && !string.IsNullOrEmpty(password))
            {
                Usuario usuario = UsuarioDL.GetUsuariosByUsernamePassword(username, password);
                if (usuario != null)
                    return usuario.IdUsuario;
            }
            return null;
        }

        public static Usuario GetUsuarioByUsername(string username)
        {
            Usuario usuario = UsuarioDL.GetUsuarioByUserName(username);
            if (usuario != null)
                usuario = GetDatosUsuario(usuario);
            return usuario;
        }

        public static List<Usuario> GetUsuariosByIdDepartamento(int idDepartamento)
        {
            return UsuarioDL.GetUsuariosByIdDepartamentof(idDepartamento);
        }

        public static List<Usuario> GetUsuariosByIdCategoria(int idCategoria)
        {
            return UsuarioDL.GetUsuariosByIdCategoria(idCategoria);
        }
    }
}
