﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GestorGastos.Model;

namespace GestorGastos.Data
{
    public class HojaDL
    {
        public static List<Hoja> GetHojasByIdUsuario(int idUsuario)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lh = from h in contexto.Hojas
                         where h.IdUsuarioCreador == idUsuario && h.IndBaja == false
                         orderby h.FechaCreacion descending
                         select h;
                return lh.ToList();
            }
        }

        public static HojaEstadoTipo GetHojaEstadoTipo(int idHojaEstadoTipo)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var le = from e in contexto.HojaEstadoTipos
                         where e.IdHojaEstadoTipo == idHojaEstadoTipo
                         select e;
                if (le.Count<HojaEstadoTipo>() > 0)
                    return le.First();
                else
                    return null;
            }
        }

        public static List<HojaEstadoTipo> GetAllHojaEstadoTipo()
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var le = from e in contexto.HojaEstadoTipos
                         select e;
                return le.ToList();
            }
        }

        public static List<Hoja> GetHojasByIdUsuarioHojaEstadoTipo(int idUsuario, int idHojaEstadoTipo)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lh = from h in contexto.Hojas
                         where h.IdUsuarioCreador == idUsuario && h.IdHojaEstadoTipo == idHojaEstadoTipo && h.IndBaja == false
                         orderby h.FechaCreacion descending
                         select h;
                return lh.ToList();
            }
        }

        public static Hoja GetHoja(int idHoja)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lh = from h in contexto.Hojas
                         where h.IdHoja == idHoja
                         orderby h.FechaCreacion descending
                         select h;
                if (lh.Count<Hoja>() > 0)
                    return lh.First();
                else
                    return null;
            }
        }

        public static bool UpdateHoja(Hoja hoja)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                contexto.Hojas.Attach(hoja);
                contexto.Entry(hoja).State = EntityState.Modified;
                return contexto.SaveChanges() > 0;
            }
        }


        public static Hoja GetHojaVigente(int idUsuario)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                if (contexto.prcGetHojaVigente(idUsuario).Count() > 0)
                    return contexto.prcGetHojaVigente(idUsuario).First();
                return null;
            }
        }

        public static int? AddHoja(Hoja hoja)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                Hoja hojaAdd = contexto.Hojas.Add(hoja);
                if (contexto.SaveChanges() == 1)
                    return hojaAdd.IdHoja;
                return null;
            }
        }

        public static int ActualizarImporteTotal(int idHoja)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                return contexto.prcUpdateHojaImporte(idHoja);
            }
        }

        public static List<Hoja> GetHojasSupervisar(int? idUsuarioSupervisor, int? idHojaEstadoTipo, string nif, string nombre)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                return contexto.prcGetHojasSupervisar(idUsuarioSupervisor, idHojaEstadoTipo, nif, nombre).OrderBy(h => h.FechaCreacion.GetValueOrDefault()).ToList<Hoja>();
            }
        }

        public static void UpdateHojasBaja(int idUsuario, bool indBaja)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                contexto.prcUpdateHojasBaja(idUsuario, indBaja);
            }
        }
    }
}
