﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GestorGastos.Model;

namespace GestorGastos.Data
{
    public class UsuarioDL
    {
        public static List<Usuario> GetUsuariosByNifNombre(string NIF, string nombre)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.NIF == NIF && u.NombreCompleto.Contains(nombre)
                         orderby u.NombreCompleto
                         select u;
                return lu.ToList();
            }
        }

        public static List<Usuario> GetUsuariosByNif(string NIF)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.NIF == NIF
                         orderby u.NombreCompleto
                         select u;
                return lu.ToList();
            }
        }

        public static List<Usuario> GetUsuariosByNombre(string nombre)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.NombreCompleto.Contains(nombre)
                         orderby u.NombreCompleto
                         select u;
                return lu.ToList();
            }
        }

        public static Usuario GetUsuario(int idUsuario)
        {
            Usuario usuario = new Usuario();
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.IdUsuario == idUsuario
                         select u;
                if (lu.Count<Usuario>() > 0)
                    return lu.First();
                else
                    return null;
            }
        }

        public static Usuario GetUsuarioSupervisor(int idUsuario)
        {
            Usuario usuario = new Usuario();
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.IdUsuario == idUsuario
                         select u;
                if (lu.Count<Usuario>() > 0)
                    return lu.First().UsuarioSupervisor;
                else
                    return null;
            }
        }


        public static List<Usuario> GetUsuariosSupervisores()
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.IdRol == (int)RolEnum.SUPERVISOR
                         orderby u.NombreCompleto
                         select u;
                //return lu.ToList();

                foreach (Usuario usuario in lu)
                {
                    if (usuario.UsuarioSupervisor != null)
                        usuario.UsuarioSupervisor = GetUsuario(usuario.UsuarioSupervisor.IdUsuario);
                }

                return lu.ToList();
            }
        }

        public static List<Usuario> GetUsuarios()
        {

            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         select u;
                return lu.ToList();
            }
        }


        public static void AddUsuario(Usuario usuario)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                if (usuario.UsuarioSupervisor != null)
                {
                    var usuarioSup = from u in contexto.Usuarios
                                     where u.IdUsuario == usuario.UsuarioSupervisor.IdUsuario
                                     select u;
                    Usuario usuarioSuperior = usuarioSup.First();
                    usuarioSuperior.UsuariosSupervisados.Add(usuario);
                    usuario.UsuarioSupervisor = usuarioSuperior;
                }

                contexto.Usuarios.Add(usuario);
                contexto.SaveChanges();
            }
        }

        public static void UpdateUsuario(Usuario usuario)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                if (usuario.UsuarioSupervisor != null)
                {
                    Usuario usuarioSupervisorBD = GetUsuarioSupervisor(usuario.IdUsuario);
                    if (usuarioSupervisorBD != null)
                    {
                        //Actualizar el supervisor 
                        contexto.prcUpdateUsuarioSupervisor(usuario.UsuarioSupervisor.IdUsuario, usuario.IdUsuario);
                        usuario.UsuarioSupervisor = null;
                    }
                    else 
                    {
                        //Insertar el supervisor 
                        contexto.prcInsertUsuarioSupervisor(usuario.UsuarioSupervisor.IdUsuario, usuario.IdUsuario);
                        usuario.UsuarioSupervisor = null;
                    }
                }
                else
                {
                    //Eliminar el supervisor 
                    int borrados = contexto.prcDeleteUsuarioSupervisor(usuario.IdUsuario);
                    usuario.UsuarioSupervisor = null;
                }

                contexto.Usuarios.Attach(usuario);
                contexto.Entry(usuario).State = EntityState.Modified;
                contexto.SaveChanges();
            }
        }


        public static void DeleteUsuario(int idUsuario)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                Usuario usuario = contexto.Usuarios.Where(u => u.IdUsuario == idUsuario).First();
                if (usuario != null)
                    usuario.IndBaja = true;

                contexto.Usuarios.Attach(usuario);
                contexto.Entry(usuario).State = EntityState.Modified;
                contexto.SaveChanges();
            }
        }



        public static Usuario GetUsuariosByUsernamePassword(string username, string password)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.Username == username && u.Password == password && u.IndBaja == false
                         select u;
                if (lu.Count<Usuario>() > 0)
                    return lu.First();
                else
                    return null;
            }
        }









        /*
       public static List<Usuario> CargarUsuarios()
       {
           using (GestorGastosBDEntities bd = new GestorGastosBDEntities())
           {
               return bd.Usuarios.ToList();
           }
       }

       public static bool Existe(int id)
       {
           using (GestorGastosBDEntities bd = new GestorGastosBDEntities())
           {
               var query = (from u in bd.Usuarios where u.IdUsuario == id select u).Count();
               if (query == 0)
                   return false;
               else
                   return true;
           }
       }

     

       public static Usuario CreateUsuario(Usuario usuario)
       {
           using (GestorGastosBDEntities bd = new GestorGastosBDEntities())
           {
               Usuario u = new Usuario();
               u.IdUsuario = usuario.IdUsuario;
               u.Nombre = usuario.Nombre;
               bd.Usuarios.Add(u);
               bd.SaveChanges();
               usuario.IdUsuario = u.IdUsuario;
           }
           return usuario;
       }
       */




        public static Usuario GetUsuarioByUserName(string username)
        {
            Usuario usuario = new Usuario();
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.Username == username
                         select u;
                if (lu.Count<Usuario>() > 0)
                    return lu.First();
                else
                    return null;
            }
        }

        public static List<Usuario> GetUsuariosByIdDepartamentof(int idDepartamento)
        {
            Usuario usuario = new Usuario();
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.IdDepartamento == idDepartamento
                         select u;
                return lu.ToList();
            }
        }

        public static List<Usuario> GetUsuariosByIdCategoria(int idCategoria)
        {
            Usuario usuario = new Usuario();
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                var lu = from u in contexto.Usuarios
                         where u.IdCategoriaProfesional == idCategoria
                         select u;
                return lu.ToList();
            }
        }

        public static void DeleteUsuariosSupervisados(int idUsuario)
        {
            using (GestorGastosBDEntities contexto = new GestorGastosBDEntities())
            {
                contexto.prcDeleteUsuariosSupervisados(idUsuario);
            }
        }
    }
}
