﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using GestorGastos.Model;
using GestorGastos.Business;

namespace GestorGastosWCFServiceLibrary
{
    public class GGMapeo
    {
        public static List<CalendarioDTO> MapearCalendarios(List<Calendario> listCalendario)
        {
            List<CalendarioDTO> result = new List<CalendarioDTO>();

            foreach (Calendario c in listCalendario)
            {
                CalendarioDTO calDTO = new CalendarioDTO();
                calDTO.IdCalendario = c.IdCalendario;
                calDTO.FechaFestiva = c.FechaFestiva;
                result.Add(calDTO);
            }
            return result;
        }

        public static List<GastoTipoDTO> MapearGastosTipo(List<GastoTipo> listGastosTipo)
        {
            List<GastoTipoDTO> result = new List<GastoTipoDTO>();

            foreach (GastoTipo g in listGastosTipo)
            {
                GastoTipoDTO gastoDTO = MapearGastoTipo(g);
                if (gastoDTO != null)
                    result.Add(gastoDTO);
            }
            return result;
        }

        public static GastoTipoDTO MapearGastoTipo(GastoTipo gastoTipo)
        {
            if (gastoTipo != null)
            {
                GastoTipoDTO gastoDTO = new GastoTipoDTO();
                gastoDTO.DescGastoTipo = gastoTipo.DescGastoTipo;
                gastoDTO.IdGastoTipo = gastoTipo.IdGastoTipo;
                gastoDTO.Importe = gastoTipo.Importe;
                gastoDTO.ImporteFestivo = gastoTipo.ImporteFestivo;
                gastoDTO.IndAdjuntarFichero = gastoTipo.IndAdjuntarFichero;
                gastoDTO.IndBaja = gastoTipo.IndBaja;
                gastoDTO.IndImporteDiferenteFestivo = gastoTipo.IndImporteDiferenteFestivo;
                gastoDTO.IndImporteLibre = gastoTipo.IndImporteLibre;
                return gastoDTO;
            }
            return null;
        }

        public static GastoTipo MapearGastoTipoDTO(GastoTipoDTO gastoTipoDTO)
        {
            if (gastoTipoDTO != null)
            {
                GastoTipo gasto = new GastoTipo();
                gasto.DescGastoTipo = gastoTipoDTO.DescGastoTipo;
                gasto.IdGastoTipo = gastoTipoDTO.IdGastoTipo;
                gasto.Importe = gastoTipoDTO.Importe;
                gasto.ImporteFestivo = gastoTipoDTO.ImporteFestivo;
                gasto.IndAdjuntarFichero = gastoTipoDTO.IndAdjuntarFichero;
                gasto.IndBaja = gastoTipoDTO.IndBaja;
                gasto.IndImporteDiferenteFestivo = gastoTipoDTO.IndImporteDiferenteFestivo;
                gasto.IndImporteLibre = gastoTipoDTO.IndImporteLibre;
                return gasto;
            }
            return null;
        }

        public static List<DepartamentoDTO> MapearDepartamentos(List<Departamento> listDepartamentos)
        {
            List<DepartamentoDTO> result = new List<DepartamentoDTO>();

            foreach (Departamento d in listDepartamentos)
            {
                DepartamentoDTO departamentoDTO = MapearDepartamento(d);
                if (departamentoDTO != null)
                    result.Add(departamentoDTO);
            }
            return result;
        }

        public static DepartamentoDTO MapearDepartamento(Departamento departamento)
        {
            if (departamento != null)
            {
                DepartamentoDTO departamentoDTO = new DepartamentoDTO();
                departamentoDTO.DescDepartamento = departamento.DescDepartamento;
                departamentoDTO.IdDepartamento = departamento.IdDepartamento;
                departamentoDTO.IndBaja = departamento.IndBaja;
                return departamentoDTO;
            }
            return null;
        }

        public static Departamento MapearDepartamentoDTO(DepartamentoDTO departamentoDTO)
        {
            if (departamentoDTO != null)
            {
                Departamento departamento = new Departamento();
                departamento.DescDepartamento = departamentoDTO.DescDepartamento;
                departamento.IdDepartamento = departamentoDTO.IdDepartamento;
                departamento.IndBaja = departamentoDTO.IndBaja;
                return departamento;
            }
            return null;
        }

        public static List<CategoriaProfesionalDTO> MapearCategorias(List<CategoriaProfesional> listCategorias)
        {
            List<CategoriaProfesionalDTO> result = new List<CategoriaProfesionalDTO>();

            foreach (CategoriaProfesional c in listCategorias)
            {
                CategoriaProfesionalDTO categoriaDTO = MapearCategoria(c);
                if (categoriaDTO != null)
                    result.Add(categoriaDTO);
            }
            return result;
        }

        public static CategoriaProfesionalDTO MapearCategoria(CategoriaProfesional categoria)
        {
            if (categoria != null)
            {
                CategoriaProfesionalDTO categoriaDTO = new CategoriaProfesionalDTO();
                categoriaDTO.DescCategoriaProfesional = categoria.DescCategoriaProfesional;
                categoriaDTO.IdCategoriaProfesional = categoria.IdCategoriaProfesional;
                categoriaDTO.IndBaja = categoria.IndBaja;
                return categoriaDTO;
            }
            return null;
        }

        public static CategoriaProfesional MapearDepartamentoDTO(CategoriaProfesionalDTO categoriaDTO)
        {
            if (categoriaDTO != null)
            {
                CategoriaProfesional categoria = new CategoriaProfesional();
                categoria.DescCategoriaProfesional = categoriaDTO.DescCategoriaProfesional;
                categoria.IdCategoriaProfesional = categoriaDTO.IdCategoriaProfesional;
                categoria.IndBaja = categoriaDTO.IndBaja;
                return categoria;
            }
            return null;
        }

        public static List<UsuarioDTO> MapearUsuarios(List<Usuario> listUsuarios)
        {
            List<UsuarioDTO> result = new List<UsuarioDTO>();

            foreach (Usuario u in listUsuarios)
            {
                UsuarioDTO usuarioDTO = MapearUsuario(u);
                if (usuarioDTO != null)
                    result.Add(usuarioDTO);
            }
            return result;
        }

        public static UsuarioDTO MapearUsuario(Usuario usuario)
        {
            if (usuario != null)
            {
                UsuarioDTO usuarioDTO = new UsuarioDTO();
                usuarioDTO.Apellido1 = usuario.Apellido1;
                usuarioDTO.Apellido2 = usuario.Apellido2;
                usuarioDTO.Email = usuario.Email;
                usuarioDTO.IdCategoriaProfesional = usuario.IdCategoriaProfesional;
                usuarioDTO.IdDepartamento = usuario.IdDepartamento;
                usuarioDTO.IdRol = usuario.IdRol;
                usuarioDTO.IdUsuario = usuario.IdUsuario;
                usuarioDTO.IndBaja = usuario.IndBaja;
                usuarioDTO.NIF = usuario.NIF;
                usuarioDTO.Nombre = usuario.Nombre;
                usuarioDTO.NombreCompleto = usuario.NombreCompleto;
                usuarioDTO.Username = usuario.Username;
                usuarioDTO.Password = usuario.Password;

                if (usuario.UsuarioSupervisor != null)
                {
                    usuarioDTO.IdUsuarioSupervisor = usuario.UsuarioSupervisor.IdUsuario;
                    usuarioDTO.NombreUsuarioSupervisor = usuario.UsuarioSupervisor.NombreCompleto;
                }

                if (usuario.IdCategoriaProfesional.HasValue)
                {
                    CategoriaProfesional categoria = CategoriaProfesionalBL.GetCategoria(usuario.IdCategoriaProfesional.Value);
                    usuarioDTO.DescCategoriaProfesional = categoria.DescCategoriaProfesional;
                }

                if (usuario.IdDepartamento.HasValue)
                {
                    Departamento departamento = DepartamentoBL.GetDepartamento(usuario.IdDepartamento.Value);
                    usuarioDTO.DescDepartamento = departamento.DescDepartamento;
                }

                if (usuario.IdRol.HasValue)
                {
                    Rol rol = RolBL.GetRol(usuario.IdRol.Value);
                    usuarioDTO.DescRol = rol.DescRol;
                }

                if (usuarioDTO.IndBaja.GetValueOrDefault())
                    usuarioDTO.Estado = "Baja";
                else
                    usuarioDTO.Estado = "Activo";

                return usuarioDTO;
            }
            return null;
        }

        public static Usuario MapearUsuarioDTO(UsuarioDTO usuarioDTO)
        {

            if (usuarioDTO != null)
            {
                Usuario usuario = new Usuario();
                usuario.Apellido1 = usuarioDTO.Apellido1;
                usuario.Apellido2 = usuarioDTO.Apellido2;
                usuario.Email = usuarioDTO.Email;
                usuario.IdCategoriaProfesional = usuarioDTO.IdCategoriaProfesional;
                usuario.IdDepartamento = usuarioDTO.IdDepartamento;
                usuario.IdRol = usuarioDTO.IdRol;
                usuario.IdUsuario = usuarioDTO.IdUsuario;
                usuario.IndBaja = usuarioDTO.IndBaja;
                usuario.NIF = usuarioDTO.NIF;
                usuario.Nombre = usuarioDTO.Nombre;
                usuario.NombreCompleto = usuarioDTO.NombreCompleto;
                usuario.Password = usuarioDTO.Password;
                usuario.Username = usuarioDTO.Username;

                if (usuarioDTO.UsuarioSupervisor != null)
                    usuario.UsuarioSupervisor = UsuarioBL.GetUsuario(usuarioDTO.UsuarioSupervisor.IdUsuario);

                /*
                if (usuarioDTO.UsuarioSupervisor != null)
                {
                    Usuario usuarioSupervisor = UsuarioBL.GetUsuario(usuarioDTO.UsuarioSupervisor.IdUsuario);
                    usuario.UsuarioSupervisor = usuarioSupervisor;
                }
               */
                /*
                List<Usuario> lstUsuario = new List<Usuario>();
                lstUsuario.Add(contexto.Usuarios.First());
                usuario.UsuariosSupervisados = lstUsuario;
                */

                /*
                if (usuario.IdRol.HasValue)
                {
                    Rol rol = RolBL.GetRol(usuario.IdRol.Value);
                    usuario.Rol = rol;
                }
                */
                return usuario;
            }
            return null;
        }

        public static List<RolDTO> MapearRoles(List<Rol> listRoles)
        {
            List<RolDTO> result = new List<RolDTO>();

            foreach (Rol r in listRoles)
            {
                RolDTO rolDTO = MapearRol(r);
                if (rolDTO != null)
                    result.Add(rolDTO);
            }
            return result;
        }

        private static RolDTO MapearRol(Rol rol)
        {
            if (rol != null)
            {
                RolDTO rolDTO = new RolDTO();
                rolDTO.DescRol = rol.DescRol;
                rolDTO.IdRol = rol.IdRol;

                return rolDTO;
            }
            return null;
        }

        public static List<HojaDTO> MapearHojas(List<Hoja> listaHojas)
        {
            List<HojaDTO> result = new List<HojaDTO>();

            foreach (Hoja h in listaHojas)
            {
                HojaDTO hojaDTO = MapearHoja(h);
                if (hojaDTO != null)
                    result.Add(hojaDTO);
            }
            return result;
        }

        public static HojaDTO MapearHoja(Hoja hoja)
        {
            if (hoja != null)
            {
                HojaDTO hojaDTO = new HojaDTO();

                hojaDTO.FechaCreacion = hoja.FechaCreacion;
                hojaDTO.DesMotivoDenegada = hoja.DesMotivoDenegada;
                hojaDTO.IdHoja = hoja.IdHoja;
                hojaDTO.IdHojaEstadoTipo = hoja.IdHojaEstadoTipo;
                hojaDTO.IdUsuarioCreador = hoja.IdUsuarioCreador;
                hojaDTO.IdUsuarioSupervisor = hoja.IdUsuarioSupervisor;
                hojaDTO.Importe = hoja.Importe;
                hojaDTO.IndBaja = hoja.IndBaja;
                hojaDTO.Referencia = hoja.Referencia;

                if (hoja.Usuario != null)
                    hojaDTO.NombreUsuarioCreador = hoja.Usuario.NombreCompleto;

                if (hoja.HojaEstadoTipo != null)
                    hojaDTO.Estado = hoja.HojaEstadoTipo.DescHojaEstadoTipo;

                if (hoja.UsuarioSupervisor != null)
                    hojaDTO.NombreUsuarioAprobador = hoja.UsuarioSupervisor.NombreCompleto;

                return hojaDTO;
            }
            return null;
        }

        public static List<HojaEstadoTipoDTO> MapearHojaEstadoTipos(List<HojaEstadoTipo> listaEstadosHoja)
        {
            List<HojaEstadoTipoDTO> result = new List<HojaEstadoTipoDTO>();

            foreach (HojaEstadoTipo h in listaEstadosHoja)
            {
                HojaEstadoTipoDTO hojaEstadoTipoDTO = MapearHojaEstadoTipo(h);
                if (hojaEstadoTipoDTO != null)
                    result.Add(hojaEstadoTipoDTO);
            }
            return result;
        }

        private static HojaEstadoTipoDTO MapearHojaEstadoTipo(HojaEstadoTipo hojaEstadoTipo)
        {
            if (hojaEstadoTipo != null)
            {
                HojaEstadoTipoDTO hojaEstadoTipoDTO = new HojaEstadoTipoDTO();
                hojaEstadoTipoDTO.DescHojaEstadoTipo = hojaEstadoTipo.DescHojaEstadoTipo;
                hojaEstadoTipoDTO.IdHojaEstadoTipo = hojaEstadoTipo.IdHojaEstadoTipo;

                return hojaEstadoTipoDTO;
            }
            return null;
        }

        public static List<GastoDTO> MapearGastos(List<Gasto> listaGastos)
        {
            List<GastoDTO> result = new List<GastoDTO>();

            foreach (Gasto g in listaGastos)
            {
                GastoDTO gastoDTO = MapearGasto(g);
                if (gastoDTO != null)
                    result.Add(gastoDTO);
            }
            return result;
        }

        public static GastoDTO MapearGasto(Gasto gasto)
        {
            if (gasto != null)
            {
                GastoDTO gastoDTO = new GastoDTO();
                gastoDTO.Fecha = gasto.Fecha;
                gastoDTO.Fichero = gasto.Fichero;
                gastoDTO.IdGasto = gasto.IdGasto;
                gastoDTO.IdGastoTipo = gasto.IdGastoTipo;
                gastoDTO.IdHoja = gasto.IdHoja;
                gastoDTO.ImporteTotal = gasto.ImporteTotal;
                gastoDTO.ImporteUnidad = gasto.ImporteUnidad;
                gastoDTO.IndBaja = gasto.IndBaja;
                gastoDTO.Unidades = gasto.Unidades;

                if (gasto.GastoTipo != null)
                    gastoDTO.DescGastoTipo = gasto.GastoTipo.DescGastoTipo;

                return gastoDTO;
            }
            return null;
        }



        public  static Gasto MapearGastoDTO(GastoDTO gastoDTO)
        {
            if (gastoDTO != null)
            {
                Gasto gasto = new Gasto();

                gasto.Fecha = gastoDTO.Fecha;
                gasto.Fichero = gastoDTO.Fichero;
                
                gasto.GastoTipo = gastoDTO.GastoTipo;
                gasto.Hoja = gastoDTO.Hoja;

                gasto.IdGasto = gastoDTO.IdGasto;
                gasto.IdGastoTipo = gastoDTO.IdGastoTipo;
                gasto.IdHoja = gastoDTO.IdHoja;

                gasto.ImporteTotal = gastoDTO.ImporteTotal;
                gasto.ImporteUnidad = gastoDTO.ImporteUnidad;
                gasto.IndBaja = gastoDTO.IndBaja;
                gasto.Unidades = gastoDTO.Unidades;

                return gasto;
            }
            return null;
        }
    }
}

