﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

using GestorGastos.Model;
using GestorGastos.Business;

namespace GestorGastosWCFServiceLibrary
{
    // NOTA: puede usar el comando "Rename" del menú "Refactorizar" para cambiar el nombre de clase "Service1" en el código y en el archivo de configuración a la vez.
    public class GestorGastosService : IGestorGastosService
    {
        #region Calendario
        public List<CalendarioDTO> ObtenerFestividades()
        {
            try
            {
                List<Calendario> listCalendario = CalendarioBL.GetFestividades();
                return GGMapeo.MapearCalendarios(listCalendario);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public bool AgregarFestividad(DateTime fecha)
        {
            try
            {
                return CalendarioBL.AgregarFestividad(fecha);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void EliminarFecha(int idCalendario)
        {
            try
            {
                CalendarioBL.EliminarFecha(idCalendario);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public bool EsFechaFestiva(DateTime fecha)
        {
            try
            {
                return CalendarioBL.EsFechaFestiva(fecha);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }
        #endregion

        #region GastoTipo
        public List<GastoTipoDTO> ObtenerGastosTipos()
        {
            try
            {
                List<GastoTipo> listGastosTipo = GastoTipoBL.GetGastosTipos();
                return GGMapeo.MapearGastosTipo(listGastosTipo);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public GastoTipoDTO ObtenerGastoTipo(int idGastoTipo)
        {
            try
            {
                GastoTipo gastoTipo = GastoTipoBL.GetGastoTipo(idGastoTipo);
                return GGMapeo.MapearGastoTipo(gastoTipo);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void ActualizarGastoTipo(GastoTipoDTO gastoTipoDTO)
        {
            try
            {
                GastoTipo gastoTipo = GGMapeo.MapearGastoTipoDTO(gastoTipoDTO);
                GastoTipoBL.ActualizarGastoTipo(gastoTipo);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void EliminarGastoTipo(int idGastoTipo)
        {
            try
            {
                GastoTipoBL.EliminarGastoTipo(idGastoTipo);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }
        #endregion

        #region Departamento
        public List<DepartamentoDTO> ObtenerDepartamentos()
        {
            try
            {
                List<Departamento> listDepartamentos = DepartamentoBL.GetDepartamentos();
                return GGMapeo.MapearDepartamentos(listDepartamentos);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public DepartamentoDTO ObtenerDepartamento(int idDepartamento)
        {
            try
            {
                Departamento departamento = DepartamentoBL.GetDepartamento(idDepartamento);
                return GGMapeo.MapearDepartamento(departamento);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void ActualizarDepartamento(DepartamentoDTO departamentoDTO)
        {
            try
            {
                Departamento departamento = GGMapeo.MapearDepartamentoDTO(departamentoDTO);
                DepartamentoBL.ActualizarDepartamento(departamento);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void EliminarDepartamento(int idDepartamento)
        {
            try
            {
                DepartamentoBL.EliminarDepartamento(idDepartamento);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }
        #endregion

        #region CategoriaProfesional
        public List<CategoriaProfesionalDTO> ObtenerCategorias()
        {
            try
            {
                List<CategoriaProfesional> listCategorias = CategoriaProfesionalBL.GetCategorias();
                return GGMapeo.MapearCategorias(listCategorias);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public CategoriaProfesionalDTO ObtenerCategoria(int idCategoria)
        {
            try
            {
                CategoriaProfesional categoria = CategoriaProfesionalBL.GetCategoria(idCategoria);
                return GGMapeo.MapearCategoria(categoria);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void ActualizarCategoria(CategoriaProfesionalDTO categoriaDTO)
        {
            try
            {
                CategoriaProfesional categoria = GGMapeo.MapearDepartamentoDTO(categoriaDTO);
                CategoriaProfesionalBL.ActualizarCategoria(categoria);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void EliminarCategoria(int idCategoria)
        {
            try
            {
                CategoriaProfesionalBL.EliminarCategoria(idCategoria);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }
        #endregion

        #region Usuario
        public List<UsuarioDTO> ObtenerUsuariosPorNIFNombre(string NIF, string nombre)
        {
            try
            {
                List<Usuario> listUsuarios = UsuarioBL.GetUsuariosByNifNombre(NIF, nombre);
                return GGMapeo.MapearUsuarios(listUsuarios);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public List<UsuarioDTO> ObtenerUsuariosPorNIF(string NIF)
        {
            try
            {
                List<Usuario> listUsuarios = UsuarioBL.GetUsuariosByNif(NIF);
                return GGMapeo.MapearUsuarios(listUsuarios);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public List<UsuarioDTO> ObtenerUsuariosPorNombre(string nombre)
        {
            try
            {
                List<Usuario> listUsuarios = UsuarioBL.GetUsuariosByNombre(nombre);
                return GGMapeo.MapearUsuarios(listUsuarios);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public UsuarioDTO ObtenerUsuario(int idUsuario)
        {
            try
            {
                Usuario usuario = UsuarioBL.GetUsuario(idUsuario);
                return GGMapeo.MapearUsuario(usuario);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public List<UsuarioDTO> ObtenerUsuariosSupervisores()
        {
            try
            {
                List<Usuario> listUsuarios = UsuarioBL.GetUsuariosSupervisores();
                return GGMapeo.MapearUsuarios(listUsuarios);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void ActualizarUsuario(UsuarioDTO usuarioDTO)
        {
            try
            {
                Usuario usuario = GGMapeo.MapearUsuarioDTO(usuarioDTO);
                UsuarioBL.ActualizarUsuario(usuario);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void RehabilitarUsuario(UsuarioDTO usuarioDTO)
        {
            try
            {
                Usuario usuario = GGMapeo.MapearUsuarioDTO(usuarioDTO);
                UsuarioBL.RehabilitarUsuario(usuario);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void InsertarUsuario(UsuarioDTO usuarioDTO)
        {
            try
            {
                Usuario usuario = GGMapeo.MapearUsuarioDTO(usuarioDTO);
                UsuarioBL.InsertarUsuario(usuario);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public void EliminarUsuario(int idUsuario)
        {
            try
            {
                UsuarioBL.DeleteUsuario(idUsuario);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public UsuarioDTO ValidarLoginUsuario(string username, string password)
        {
            try
            {
                int? idUsuario = UsuarioBL.IsLoginUsuario(username, password);

                if (idUsuario.HasValue)
                    return ObtenerUsuario(idUsuario.Value);
                else
                    return null;
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public UsuarioDTO ObtenerUsuarioPorUsername(string username)
        {
            try
            {
                Usuario usuario = UsuarioBL.GetUsuarioByUsername(username);
                return GGMapeo.MapearUsuario(usuario);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }
        #endregion

        #region Rol
        public List<RolDTO> ObtenerRoles()
        {
            try
            {
                List<Rol> listRoles = RolBL.GetRoles();
                return GGMapeo.MapearRoles(listRoles);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }
        #endregion

        #region Hoja
        public List<HojaDTO> ObtenerHojasUsuario(int idUsuario)
        {
            try
            {
                List<Hoja> listaHojas = HojaBL.GetHojasByIdUsuario(idUsuario);
                return GGMapeo.MapearHojas(listaHojas);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public List<HojaEstadoTipoDTO> ObtenerHojaEstadoTipos()
        {
            try
            {
                List<HojaEstadoTipo> listaEstadosHoja = HojaBL.GetAllHojaEstadoTipo();
                return GGMapeo.MapearHojaEstadoTipos(listaEstadosHoja);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public List<HojaDTO> ObtenerHojasUsuarioByEstado(int idUsuario, int idHojaEstadoTipo)
        {
            try
            {
                List<Hoja> listaHojas = HojaBL.GetHojasByIdUsuarioHojaEstadoTipo(idUsuario, idHojaEstadoTipo);
                return GGMapeo.MapearHojas(listaHojas);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public HojaDTO ObtenerHoja(int idHoja)
        {
            try
            {
                Hoja hoja = HojaBL.GetHoja(idHoja);
                return GGMapeo.MapearHoja(hoja);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public HojaDTO ObtenerHojaPorUsuario(int idHoja, int idUsuario)
        {
            try
            {
                Hoja hoja = HojaBL.GetHoja(idHoja);
                HojaDTO result = GGMapeo.MapearHoja(hoja);
                result.PuedeEliminarHoja = HojaBL.PuedeEliminarHojaByUsuario(idHoja, idUsuario);
                result.PuedeSupervisarHoja = HojaBL.PuedeSupervisarHojaByUsuario(idHoja, idUsuario);
                return result;
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public bool EliminarHoja(int idHoja)
        {
            try
            {
                return HojaBL.EliminarHoja(idHoja);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public bool AprobarHoja(int idHoja, int idUsuarioSupervisor)
        {
            try
            {
                return HojaBL.AprobarHoja(idHoja, idUsuarioSupervisor);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public bool DenegarHoja(int idHoja, string motivo, int idUsuarioSupervisor)
        {
            try
            {
                return HojaBL.DenegarHoja(idHoja, motivo, idUsuarioSupervisor);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        /// <summary>
        /// Obtiene la lista de hojas para los supervisores según los parámetros de búsqueda
        /// </summary>
        /// <param name="idUsuarioSupervisor"></param>
        /// <param name="idHojaEstadoTipo"></param>
        /// <param name="nif"></param>
        /// <param name="nombre"></param>
        /// <returns></returns>
        public List<HojaDTO> ObtenerHojasASupervisar(int? idUsuarioSupervisor, int? idHojaEstadoTipo, string nif, string nombre)
        {
            try
            {
                List<Hoja> lstHojasSupervisar = HojaBL.GetHojasSupervisar(idUsuarioSupervisor, idHojaEstadoTipo, nif, nombre);
                return GGMapeo.MapearHojas(lstHojasSupervisar);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        /// <summary>
        /// Devuelve si existe la hoja del mes actual abierta del usuario indicado
        /// </summary>
        /// <param name="idUsuario"></param>
        /// <returns></returns>
        public HojaDTO ObtenerHojaVigente(int idUsuario)
        {
            try
            {
                Hoja hoja = HojaBL.GetHojaVigente(idUsuario);
                return GGMapeo.MapearHoja(hoja);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }
        #endregion

        #region Gasto
        public List<GastoDTO> ObtenerGastosDeHoja(int idHoja)
        {
            try
            {
                List<Gasto> listaGastos = GastoBL.GetGastosByIdHoja(idHoja);
                return GGMapeo.MapearGastos(listaGastos);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public GastoDTO ObtenerGastoPorUsuario(int idGasto, int idUsuario)
        {
            try
            {
                Gasto gasto = GastoBL.GetGasto(idGasto);
                GastoDTO result = GGMapeo.MapearGasto(gasto);
                result.PuedeEditarGasto = GastoBL.PuedeEditarGastoByUsuario(idGasto, idUsuario);
                return result;
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        /// <summary>
        /// Crear o actualizar el gasto, si no hay hoja vigente se crea la hoja
        /// </summary>
        /// <param name="gastoDTO"></param>
        /// <param name="idUsuario"></param>
        public void ActualizarGasto(GastoDTO gastoDTO, int idUsuario)
        {
            try
            {
                Gasto gasto = GGMapeo.MapearGastoDTO(gastoDTO);
                GastoBL.ActualizarGasto(gasto, idUsuario);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }

        public bool EliminarGasto(int idGasto)
        {
            try
            {
                return GastoBL.EliminarGasto(idGasto);
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                    throw ex.InnerException;
                else
                    throw ex;
            }
        }
        #endregion
    }
}
